@@ -2191,9 +2191,8 @@ inRecurrenceExceptionsForEvent: (iCalEvent *) theEvent
       //
       else if ([event userIsAttendee: ownerUser] && [self _shouldScheduleEvent: [event userAsAttendee: ownerUser]])
         {
-          [self sendIMIPReplyForEvent: event
-                                 from: ownerUser
-                                   to: [event organizer]];
+          [self sendResponseToOrganizer: event
+                                   from: ownerUser];
         }
       	      
       [self sendReceiptEmailForObject: event
@@ -2295,7 +2294,8 @@ inRecurrenceExceptionsForEvent: (iCalEvent *) theEvent
       //      [react on SENT-BY as someone else is acting for the organizer]
       //
       //
-      if ([[newEvent attendees] count] || [[oldEvent attendees] count])
+      int newCount = [[newEvent attendees] count], oldCount = [[oldEvent attendees] count];
+      if (newCount > 0 || oldCount > 0)
         {
           BOOL userIsOrganizer;
 
