@@ -1,6 +1,6 @@
 /* SOGoCalendarComponent.m - this file is part of SOGo
  *
- * Copyright (C) 2006-2019 Inverse inc.
+ * Copyright (C) 2006-2021 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -234,7 +234,7 @@
 
   tags = [NSArray arrayWithObjects: @"DTSTAMP", @"DTSTART", @"DTEND", @"DUE", @"EXDATE", @"EXRULE", @"RRULE", @"RECURRENCE-ID", nil];
   uid = [[component uid] asCryptedPassUsingScheme: @"ssha256"
-                                         withSalt: [[settings userSalt] dataUsingEncoding: NSASCIIStringEncoding]
+                                         withSalt: [[settings userPublicSalt] dataUsingEncoding: NSASCIIStringEncoding]
                                       andEncoding: encHex
                                           keyPath: nil];
 
@@ -765,10 +765,8 @@
   headerMap = [NGMutableHashMap hashMapWithCapacity: 3];
   [headerMap setObject: @"urn:content-classes:calendarmessage"  forKey: @"Content-Class"];
   [headerMap setObject: header forKey: @"content-type"];
-  [headerMap setObject: @"quoted-printable"
-                forKey: @"content-transfer-encoding"];
   bodyPart = [NGMimeBodyPart bodyPartWithHeader: headerMap];
-  [bodyPart setBody: [objectData dataByEncodingQuotedPrintable]];
+  [bodyPart setBody: objectData];
 
   return bodyPart;
 }
@@ -1095,11 +1093,11 @@
 //
 //
 //
-- (void) sendReceiptEmailForObject: (iCalRepeatableEntityObject *) object
+- (void) sendReceiptEmailForObject: (iCalEntityObject *) object
 		    addedAttendees: (NSArray *) theAddedAttendees
 		  deletedAttendees: (NSArray *) theDeletedAttendees
 		  updatedAttendees: (NSArray *) theUpdatedAttendees
-			 operation: (SOGoEventOperation) theOperation
+			 operation: (SOGoComponentOperation) theOperation
 {
   NSString *calendarName, *mailDate, *mailText, *fullSenderEmail, *senderEmail, *fullRecipientEmail, *recipientEmail;
   NSDictionary *senderIdentity, *recipientIdentity;
