@@ -1,6 +1,6 @@
 /* SOGoContactSourceFolder.m - this file is part of SOGo
  *
- * Copyright (C) 2006-2016 Inverse inc.
+ * Copyright (C) 2006-2021 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#import <Foundation/NSAutoreleasePool.h>
 #import <Foundation/NSValue.h>
 
 #import <NGObjWeb/NSException+HTTP.h>
@@ -136,8 +137,14 @@
   NSArray *type;
 
   resourceType = [NSMutableArray arrayWithArray: [super davResourceType]];
+
+  // CARDDAV:addressbook
+  // https://datatracker.ietf.org/doc/html/rfc6352#section-5.2
   type = [NSArray arrayWithObjects: @"addressbook", XMLNS_CARDDAV, nil];
   [resourceType addObject: type];
+
+  // CARDDAV:directory
+  // https://datatracker.ietf.org/doc/html/draft-daboo-carddav-directory-gateway-02#section-4.1
   type = [NSArray arrayWithObjects: @"directory", XMLNS_CARDDAV, nil];
   [resourceType addObject: type];
 
@@ -345,7 +352,7 @@
     {
       [newRecord setObject: @"vcard" forKey: @"c_component"];
     }
-  
+
   // Custom attribute for resource lookups. See LDAPSource.m.
   data = [oldRecord objectForKey: @"isResource"];
   if (data)
@@ -369,7 +376,7 @@
   if ([recordSource conformsToProtocol: @protocol (SOGoDNSource)] &&
       [[(NSObject <SOGoDNSource>*) recordSource MSExchangeHostname] length])
     [newRecord setObject: [NSNumber numberWithInt: 1] forKey: @"isMSExchange"];
-  
+
   return newRecord;
 }
 
@@ -521,12 +528,15 @@
   currentValue = values;
   while (*currentProperty)
     {
+      *values = nil;
       methodSel = SOGoSelectorForPropertyGetter (*currentProperty);
       if (methodSel && [ldifEntry respondsToSelector: methodSel])
-        *currentValue = [[ldifEntry performSelector: methodSel]
-                          safeStringByEscapingXMLString];
+        {
+          *currentValue = [[ldifEntry performSelector: methodSel]
+                            safeStringByEscapingXMLString];
+          currentValue++;
+        }
       currentProperty++;
-      currentValue++;
     }
 
 //    NSLog (@"/_properties:ofObject:: %@", [NSDate date]);
@@ -641,14 +651,16 @@
                        matchingURLs: (id <DOMNodeList>) refs
                          toResponse: (WOResponse *) response
 {
+  NSAutoreleasePool *pool;
   NSObject <DOMElement> *element;
   NSString *url, *baseURL, *cname, *domain;
   NSString **propertiesArray;
   NSMutableString *buffer;
   NSDictionary *object;
+  id connection;
 
   unsigned int count, max, propertiesCount;
-  
+
   baseURL = [self davURLAsString];
 #warning review this when fixing http://www.scalableogo.org/bugs/view.php?id=276
   if (![baseURL hasSuffix: @"/"])
@@ -658,14 +670,18 @@
   propertiesCount = [properties count];
 
   max = [refs length];
-  buffer = [NSMutableString stringWithCapacity: max*512];
+  buffer = [[NSMutableString alloc] initWithCapacity: max*512];
   domain = [[context activeUser] domain];
+  connection = [source connection];
+  pool = [[NSAutoreleasePool alloc] init];
   for (count = 0; count < max; count++)
     {
       element = [refs objectAtIndex: count];
       url = [[[element firstChild] nodeValue] stringByUnescapingURL];
       cname = [self _deduceObjectNameFromURL: url fromBaseURL: baseURL];
-      object = [source lookupContactEntry: cname inDomain: domain];
+      object = [source lookupContactEntry: cname
+				 inDomain: domain
+			  usingConnection: connection];
       if (object)
         [self appendObject: object
                 properties: propertiesArray
@@ -675,8 +691,15 @@
       else
         [self appendMissingObjectRef: url
                             toBuffer: buffer];
+      if (count > 0 && count % 10 == 0)
+	{
+	  RELEASE(pool);
+	  pool = [[NSAutoreleasePool alloc] init];
+	}
     }
+  RELEASE(pool);
   [response appendContentString: buffer];
+  RELEASE(buffer);
 //   NSLog (@"/adding properties with url");
 
   NSZoneFree (NULL, propertiesArray);
