@@ -20,6 +20,7 @@
   02111-1307, USA.
 */
 
+#import <Foundation/NSAutoreleasePool.h>
 #import <Foundation/NSValue.h>
 #import <Foundation/NSFileHandle.h>
 
@@ -78,8 +79,7 @@
 
 static NSString *defaultUserID =  @"anyone";
 
-static NSComparisonResult
-_compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
+static NSComparisonResult _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 {
   static NSNumber *zeroNumber = nil;
   NSNumber *modseq1, *modseq2;
@@ -100,6 +100,22 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   return [modseq1 compare: modseq2];
 }
 
+static NSInteger _compareFetchResultsByUID (id entry1, id entry2, NSArray *uids)
+{
+  NSString *uid1, *uid2;
+  NSUInteger pos1, pos2;
+
+  uid1 = [entry1 objectForKey: @"uid"];
+  uid2 = [entry2 objectForKey: @"uid"];
+  pos1 = [uids indexOfObject: uid1];
+  pos2 = [uids indexOfObject: uid2];
+
+  if (pos1 > pos2)
+    return NSOrderedDescending;
+  else
+    return NSOrderedAscending;
+}
+
 @interface NGImap4Connection (PrivateMethods)
 
 - (NSString *) imap4FolderNameForURL: (NSURL *) url;
@@ -301,8 +317,7 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 {
   NSException *error;
   SOGoMailFolder *inbox;
-  NSURL *destURL;
-  NSString *path, *newName;
+  NSString *path, *newName, *destName;
   NGImap4Client *client;
 
   if ([theNewName length] > 0)
@@ -324,22 +339,16 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 
 	  // If new name contains the path - dont't need to add
           if ([newName rangeOfString: @"/"].location == NSNotFound)
-            destURL = [[NSURL alloc] initWithScheme: [imap4URL scheme]
-                                               host: [imap4URL host]
-                                               path: [NSString stringWithFormat: @"%@%@",
-                                                               path, newName]];
+            destName = [NSString stringWithFormat: @"%@%@",
+                                 path, newName];
           else
-            destURL = [[NSURL alloc] initWithScheme: [imap4URL scheme]
-                                               host: [imap4URL host]
-                                               path: [NSString stringWithFormat: @"%@",
-                                                               newName]];
-          [destURL autorelease];
-          error = [imap4 moveMailboxAtURL: imap4URL
-                                    toURL: destURL];
+            destName = newName;
+          error = [imap4 moveMailbox: [imap4URL path]
+                                  to: destName];
           if (!error)
             {
               // We unsubscribe to the old one, and subscribe back to the new one
-              [client subscribe: [destURL path]];
+              [client subscribe: destName];
               [client unsubscribe: [imap4URL path]];
 
               ASSIGN (imap4URL, nil);
@@ -627,6 +636,27 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 - (WOResponse *) copyUIDs: (NSArray *) uids
 		 toFolder: (NSString *) destinationFolder
 		inContext: (id) localContext
+{
+  return [self moveUIDs: uids
+               toFolder: destinationFolder
+              inContext: localContext
+               keepCopy: YES];
+}
+
+- (WOResponse *) moveUIDs: (NSArray *) uids
+		 toFolder: (NSString *) destinationFolder
+		inContext: (id) localContext
+{
+  return [self moveUIDs: uids
+               toFolder: destinationFolder
+              inContext: localContext
+               keepCopy: NO];
+}
+
+- (WOResponse *) moveUIDs: (NSArray *) uids
+		 toFolder: (NSString *) destinationFolder
+		inContext: (id) localContext
+                 keepCopy: (BOOL) copy
 {
   NSArray *folders;
   NSString *currentFolderName, *currentAccountName, *destinationAccountName;
@@ -657,22 +687,40 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
           if ([destinationAccountName isEqualToString: currentAccountName])
             {
               // We make sure the destination IMAP folder exist, if not, we create it.
-              result = [[client status: imapDestinationFolder
-                                 flags: [NSArray arrayWithObject: @"UIDVALIDITY"]]
-                         objectForKey: @"result"];
-              if (![result boolValue])
+              result = [[[client list: @"" pattern: imapDestinationFolder] objectForKey: @"list"] objectForKey: imapDestinationFolder];
+              if (!result)
                 result = [[self imap4Connection] createMailbox: imapDestinationFolder
                                                          atURL: [[self mailAccountFolder] imap4URL]];
               if (!result || [result boolValue])
-                result = [client copyUids: uids toFolder: imapDestinationFolder];
+                {
+                  if (copy || ![[self mailAccountFolder] supportsMove])
+                    result = [client copyUids: uids toFolder: imapDestinationFolder];
+                  else
+                    result = [client moveUids: uids toFolder: imapDestinationFolder];
+                }
 
               if ([[result valueForKey: @"result"] boolValue])
-                result = nil;
+                {
+                  result = nil;
+                  if (!copy && ![[self mailAccountFolder] supportsMove])
+                    {
+                      // Server doesn't support MOVE -- delete messages
+                      result = [client storeFlags: [NSArray arrayWithObject: @"Deleted"]
+                                          forUIDs: uids addOrRemove: YES];
+                      if ([[result valueForKey: @"result"] boolValue])
+                        {
+                          [self markForExpunge];
+                          result = nil;
+                        }
+                    }
+                }
               else
-                result = [NSException exceptionWithHTTPStatus: 500
-                                                       reason: [[[result objectForKey: @"RawResponse"]
-                                                                  objectForKey: @"ResponseResult"]
-                                                                 objectForKey: @"description"]];
+                {
+                  result = [NSException exceptionWithHTTPStatus: 500
+                                                         reason: [[[result objectForKey: @"RawResponse"]
+                                                                    objectForKey: @"ResponseResult"]
+                                                                   objectForKey: @"description"]];
+                }
             }
           else
             {
@@ -750,36 +798,6 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   return result;
 }
 
-- (WOResponse *) moveUIDs: (NSArray *) uids
-		 toFolder: (NSString *) destinationFolder
-		inContext: (id) localContext
-{
-  id result;
-  NGImap4Client *client;
-
-  client = [[self imap4Connection] client];
-  if (client)
-    {
-      result = [self copyUIDs: uids toFolder: destinationFolder inContext: localContext];
-      if (![result isNotNull])
-        {
-          result = [client storeFlags: [NSArray arrayWithObject: @"Deleted"]
-                              forUIDs: uids addOrRemove: YES];
-          if ([[result valueForKey: @"result"] boolValue])
-            {
-              [self markForExpunge];
-              result = nil;
-            }
-        }
-    }
-  else
-    result = [NSException exceptionWithName: @"SOGoMailException"
-                                     reason: @"IMAP connection is invalid"
-                                   userInfo: nil];
-
-  return result;
-}
-
 - (WOResponse *) markMessagesAsJunkOrNotJunk: (NSArray *) uids
                                         junk: (BOOL) isJunk
 {
@@ -1306,6 +1324,8 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
       switch ([imapAcls characterAtIndex: count])
 	{
 	case 'l':
+	  [SOGoAcls addObjectUniquely: SOGoRole_FolderViewer];
+          break;
 	case 'r':
 	  [SOGoAcls addObjectUniquely: SOGoRole_ObjectViewer];
 	  break;
@@ -1388,11 +1408,10 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   acls = [sogoAcls objectEnumerator];
   while ((currentAcl = [acls nextObject]))
     {
-      if ([currentAcl isEqualToString: SOGoRole_ObjectViewer])
-	{
-	  [imapAcls appendFormat: @"lr"];
-	  character = 0;
-	}
+      if ([currentAcl isEqualToString: SOGoRole_FolderViewer])
+	character = 'l';
+      else if ([currentAcl isEqualToString: SOGoRole_ObjectViewer])
+	character = 'r';
       else if ([currentAcl isEqualToString: SOGoMailRole_SeenKeeper])
 	character = 's';
       else if ([currentAcl isEqualToString: SOGoMailRole_Writer])
@@ -1757,6 +1776,8 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
                              forKey: @"{urn:schemas:mailheader:}references"];
       [davIMAPFieldsTable setObject: @"BODY[HEADER.FIELDS (SUBJECT)]"
                              forKey: @"{DAV:}displayname"];
+      [davIMAPFieldsTable setObject: @"RFC822.SIZE"
+                             forKey: @"{DAV:}getcontentlength"];
       [davIMAPFieldsTable setObject: @"BODY[HEADER.FIELDS (TO)]"
                              forKey: @"{urn:schemas:mailheader:}to"];
     }
@@ -1846,7 +1867,7 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 {
   NGImap4Client *client;
   NSDictionary *response;
-  NSArray *messages, *values = nil;
+  NSArray *uids, *results, *sortedResults = nil;
   NSString *resultKey;
 
   client = [[self imap4Connection] client];
@@ -1866,15 +1887,17 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 
    if ([[response objectForKey: @"result"] boolValue])
      {
-       messages = [response objectForKey: resultKey];
-       if ([messages count] > 0)
+       uids = [response objectForKey: resultKey];
+       if ([uids count] > 0)
          {
-           response = [client fetchUids: messages parts: properties];
-           values = [response objectForKey: @"fetch"];
+           response = [client fetchUids: uids parts: properties];
+           results = [response objectForKey: @"fetch"];
+           sortedResults = (NSArray *)[results sortedArrayUsingFunction: _compareFetchResultsByUID
+                                                                context: (void *)uids];
          }
      }
 
-  return values;
+  return sortedResults;
 }
 
 - (NSArray *) _davPropstatsWithProperties: (NSArray *) davProperties
@@ -1938,10 +1961,10 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
               fromMessages: (NSArray *) messages
                 toResponse: (WOResponse *) response
 {
-  NSDictionary *davElement;
+  NSDictionary *davElement, *message;
   NSArray *propstats;
   NSMutableArray *all;
-  NSString *message, *davString;
+  NSString *davString;
   SEL *selectors;
   int max, count;
 
@@ -1949,19 +1972,17 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   selectors = NSZoneMalloc (NULL, sizeof (max * sizeof (SEL)));
 
   for (count = 0; count < max; count++)
-    selectors[count]
-      = SOGoSelectorForPropertyGetter ([properties objectAtIndex: count]);
+    selectors[count] = SOGoSelectorForPropertyGetter ([properties objectAtIndex: count]);
 
   max = [messages count];
   all = [NSMutableArray array];
   for (count = 0; count < max; count++)
     {
-      message = [[messages objectAtIndex: count] stringValue];
+      message = [messages objectAtIndex: count];
       propstats = [self _davPropstatsWithProperties: properties
                                  andMethodSelectors: selectors
-                                         fromMessage: message];
-      davElement = davElementWithContent (@"response", XMLNS_WEBDAV, 
-                                          propstats);
+                                        fromMessage: [[message objectForKey: @"uid"] stringValue]];
+      davElement = davElementWithContent (@"response", XMLNS_WEBDAV, propstats);
 
       [all addObject: davElement];
     }
@@ -2035,14 +2056,13 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   properties = [self parseDAVRequestedProperties: propElement];
   filterElement = [documentElement firstElementWithTag: @"mail-filters"
                                            inNamespace: XMLNS_INVERSEDAV];
-  searchQualifier = [EOQualifier
-                      qualifierFromMailDAVMailFilters: filterElement];
+  searchQualifier = [EOQualifier qualifierFromMailDAVMailFilters: filterElement];
   sortElement = (NGDOMNodeWithChildren *) [documentElement
                                             firstElementWithTag: @"sort"
                                                     inNamespace: XMLNS_INVERSEDAV];
   sortOrderings = [self _sortOrderingsFromSortElement: sortElement];
 
-  messages = [self _fetchMessageProperties: [properties allKeys]
+  messages = [self _fetchMessageProperties: [properties allValues]
                          matchingQualifier: searchQualifier
                           andSortOrderings: sortOrderings];
   [self _appendProperties: [properties allKeys]
@@ -2266,16 +2286,33 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
                           matchingSyncToken: (NSString *) theSyncToken
                                    fromDate: (NSCalendarDate *) theStartDate
                                 initialLoad: (BOOL) initialLoadInProgress
+{
+  return [self syncTokenFieldsWithProperties: theProperties
+                           matchingSyncToken: theSyncToken
+                                    fromDate: theStartDate
+                                 initialLoad: initialLoadInProgress
+                                sortOrdering: nil
+                                    threaded: NO];
+}
+
+- (NSArray *) syncTokenFieldsWithProperties: (NSDictionary *) theProperties
+                          matchingSyncToken: (NSString *) theSyncToken
+                                   fromDate: (NSCalendarDate *) theStartDate
+                                initialLoad: (BOOL) initialLoadInProgress
+                               sortOrdering: (id) theSortOrdering
+                                   threaded: (BOOL) isThreaded
 {
   EOQualifier *searchQualifier;
+  NSAutoreleasePool *pool;
   NSMutableArray *allTokens;
   NSArray *a, *uids;
   NSDictionary *d;
-  id fetchResults;
+  id fetchResults, sortedResults;
 
   int highestmodseq = 0, i;
 
   allTokens = [NSMutableArray array];
+  pool = [[NSAutoreleasePool alloc] init];
 
   if (![theSyncToken isEqualToString: @"-1"])
     {
@@ -2285,7 +2322,6 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   
   // We first make sure QRESYNC is enabled
   [[self imap4Connection] enableExtensions: [NSArray arrayWithObject: @"QRESYNC"]];
-   
 
   // We fetch new messages and modified messages
   if (highestmodseq)
@@ -2323,28 +2359,36 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 
   // we fetch modified or added uids
   uids = [self fetchUIDsMatchingQualifier: searchQualifier
-                             sortOrdering: nil];
+                             sortOrdering: theSortOrdering];
 
   fetchResults = [(NSDictionary *)[self fetchUIDs: uids
                                             parts: [NSArray arrayWithObjects: @"modseq", @"flags", nil]]
                      objectForKey: @"fetch"];
   
-  /* NOTE: we sort items manually because Cyrus does not properly sort
-     entries with a MODSEQ of 0 */
-  fetchResults
-    = [fetchResults sortedArrayUsingFunction: _compareFetchResultsByMODSEQ
-                                     context: NULL];
+  if (theSortOrdering == nil)
+    {
+      /* NOTE: we sort items manually because Cyrus does not properly sort
+         entries with a MODSEQ of 0 */
+      sortedResults = [fetchResults sortedArrayUsingFunction: _compareFetchResultsByMODSEQ
+                                                    context: NULL];
+    }
+  else
+    {
+      sortedResults = [fetchResults sortedArrayUsingFunction: _compareFetchResultsByUID
+                                                    context: uids];
+    }
 
-  for (i = 0; i < [fetchResults count]; i++)
-    { 
-      if ([[[fetchResults objectAtIndex: i] objectForKey: @"flags"] containsObject: @"deleted"] && initialLoadInProgress)
+  for (i = 0; i < [sortedResults count]; i++)
+    {
+      if ([[[sortedResults objectAtIndex: i] objectForKey: @"flags"] containsObject: @"deleted"] && initialLoadInProgress)
         continue;
 
-      d = [NSDictionary dictionaryWithObject: ([[[fetchResults objectAtIndex: i] objectForKey: @"flags"] containsObject: @"deleted"]) ? [NSNull null] : [[fetchResults objectAtIndex: i] objectForKey: @"modseq"]
-                                      forKey: [[[fetchResults objectAtIndex: i] objectForKey: @"uid"] stringValue]];
+      d = [NSDictionary dictionaryWithObject: ([[[sortedResults objectAtIndex: i] objectForKey: @"flags"] containsObject: @"deleted"]) ? [NSNull null] : [[sortedResults objectAtIndex: i] objectForKey: @"modseq"]
+                                      forKey: [[[sortedResults objectAtIndex: i] objectForKey: @"uid"] stringValue]];
       [allTokens addObject: d];
     }
   
+  [pool release];
 
   // We fetch deleted ones
   if (highestmodseq == 0)
