@@ -45,6 +45,9 @@
 @interface SOGoMailBrazilianPortugueseReply : SOGoMailReply
 @end
 
+@interface SOGoMailBulgarianReply : SOGoMailReply
+@end
+
 @interface SOGoMailCatalanReply : SOGoMailReply
 @end
 
@@ -105,6 +108,9 @@
 @interface SOGoMailMacedonianReply : SOGoMailReply
 @end
 
+@interface SOGoMailMontenegrinReply : SOGoMailReply
+@end
+
 @interface SOGoMailNorwegianBokmalReply : SOGoMailReply
 @end
 
@@ -126,6 +132,9 @@
 @interface SOGoMailSerbianReply : SOGoMailReply
 @end
 
+@interface SOGoMailSerbianLatinReply : SOGoMailReply
+@end
+
 @interface SOGoMailSlovakReply : SOGoMailReply
 @end
 
