@@ -1,6 +1,6 @@
 /* LDAPSource.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2019 Inverse inc.
+ * Copyright (C) 2007-2021 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -437,9 +437,11 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
 //
 //
 //
-- (NGLdapConnection *) _ldapConnection
+- (id) connection
 {
   NGLdapConnection *ldapConnection;
+  NSString *value, *key;
+  SOGoCache *cache;
 
   NS_DURING
     {
@@ -459,7 +461,20 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
           if (!_schema)
             {
               _schema = [LDAPSourceSchema new];
-              [_schema readSchemaFromConnection: ldapConnection];
+              cache = [SOGoCache sharedCache];
+              key = [NSString stringWithFormat: @"schema:%@", _sourceID];
+              value = [cache valueForKey: key];
+
+              if (value)
+                {
+                  [_schema setSchema: (NSMutableDictionary *)[value objectFromJSONString]];
+                }
+              else
+                {
+                  // We go check in the LDAP directory
+                  [_schema readSchemaFromConnection: ldapConnection];
+                  [cache setValue: [_schema jsonRepresentation] forKey: key];
+                }
             }
         }
       else
@@ -477,6 +492,11 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
   return ldapConnection;
 }
 
+- (NGLdapConnection *) _ldapConnection
+{
+  return (NGLdapConnection *)[self connection];
+}
+
 - (NSString *) domain
 {
   return _domain;
@@ -718,10 +738,15 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
                   }
                 else if (_passwordPolicy)
                   {
-                    didChange = [bindConnection changePasswordAtDn: userDN
-                                                       oldPassword: oldPassword
-                                                       newPassword: newPassword
-                                                              perr: (void *)perr];
+                    if ([bindConnection bindWithMethod: @"simple"
+                                                binddn: _sourceBindDN
+                                           credentials: _sourceBindPassword])
+                      {
+                        didChange = [bindConnection changePasswordAtDn: userDN
+                                                           oldPassword: oldPassword
+                                                           newPassword: newPassword
+                                                                  perr: (void *)perr];
+                      }
                   }
                 else
                   {
@@ -740,12 +765,20 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
 
                     if (encryptedPass != nil)
                       {
-                        *perr = PolicyNoError;
-                        didChange = [self _ldapModifyAttribute: @"userPassword"
-                                                     withValue: encryptedPass
-                                                        userDN: userDN
-                                                      password: oldPassword
-                                                    connection: bindConnection];
+                        if ([bindConnection bindWithMethod: @"simple"
+                                                    binddn: userDN
+                                               credentials: oldPassword])
+                          {
+                            didChange = [self _ldapModifyAttribute: @"userPassword"
+                                                         withValue: encryptedPass
+                                                            userDN: userDN
+                                                          password: oldPassword
+                                                        connection: bindConnection];
+                            if (didChange)
+                              {
+                                *perr = PolicyNoError;
+                              }
+                          }
                       }
                   }
 
@@ -1255,11 +1288,13 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
                        withCriteria: (NSArray *) criteria
                            inDomain: (NSString *) theDomain
 {
+  NSAutoreleasePool *pool;
   NGLdapConnection *ldapConnection;
   NGLdapEntry *currentEntry;
   NSEnumerator *entries;
   NSMutableArray *contacts;
   EOQualifier *qualifier;
+  unsigned int i;
 
   contacts = [NSMutableArray array];
 
@@ -1280,20 +1315,33 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
         entries = [ldapConnection deepSearchAtBaseDN: _baseDN
                                            qualifier: qualifier
                                           attributes: _lookupFields];
+
+      i = 0;
+      pool = [NSAutoreleasePool new];
       while ((currentEntry = [entries nextObject]))
-        [contacts addObject:
-                    [self _convertLDAPEntryToContact: currentEntry]];
+        {
+          [contacts addObject:
+                      [self _convertLDAPEntryToContact: currentEntry]];
+          i++;
+          if (i % 10 == 0)
+            {
+              [pool release];
+              pool = [NSAutoreleasePool new];
+            }
+        }
+      [pool release];
     }
 
   return contacts;
 }
 
 - (NGLdapEntry *) _lookupLDAPEntry: (EOQualifier *) theQualifier
+		   usingConnection: (id) connection
 {
   NGLdapConnection *ldapConnection;
   NSEnumerator *entries;
 
-  ldapConnection = [self _ldapConnection];
+  ldapConnection = (NGLdapConnection *)connection;
 
   if ([_scope caseInsensitiveCompare: @"BASE"] == NSOrderedSame)
     entries = [ldapConnection baseSearchAtBaseDN: _baseDN
@@ -1311,8 +1359,15 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
   return [entries nextObject];
 }
 
+- (NGLdapEntry *) _lookupLDAPEntry: (EOQualifier *) theQualifier
+{
+  return [self _lookupLDAPEntry: theQualifier
+		usingConnection: [self _ldapConnection]];
+}
+
 - (NSDictionary *) lookupContactEntry: (NSString *) theID
                              inDomain: (NSString *) theDomain
+		      usingConnection: (id) connection
 {
   NGLdapEntry *ldapEntry;
   EOQualifier *qualifier;
@@ -1326,7 +1381,8 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
       s = [NSString stringWithFormat: @"(%@='%@')",
                     _IDField, SafeLDAPCriteria(theID)];
       qualifier = [EOQualifier qualifierWithQualifierFormat: s];
-      ldapEntry = [self _lookupLDAPEntry: qualifier];
+      ldapEntry = [self _lookupLDAPEntry: qualifier
+			 usingConnection: connection];
       if (ldapEntry)
         ldifRecord = [self _convertLDAPEntryToContact: ldapEntry];
     }
@@ -1334,6 +1390,14 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
   return ldifRecord;
 }
 
+- (NSDictionary *) lookupContactEntry: (NSString *) theID
+                             inDomain: (NSString *) theDomain
+{
+  return [self lookupContactEntry: theID
+			 inDomain: theDomain
+		  usingConnection: [self _ldapConnection]];
+}
+
 - (NSDictionary *) lookupContactEntryWithUIDorEmail: (NSString *) uid
                                            inDomain: (NSString *) theDomain
 {
@@ -2028,12 +2092,12 @@ _makeLDAPChanges (NGLdapConnection *ldapConnection,
 
 - (NSArray *) membersForGroupWithUID: (NSString *) uid
 {
-  NSMutableArray *dns, *uids, *logins;
+  NSMutableArray *dns, *uids;
   NSString *dn, *login;
   SOGoUserManager *um;
   NSDictionary *d, *contactInfos;
   SOGoUser *user;
-  NSArray *o, *users;
+  NSArray *o, *subusers, *logins;
   NSAutoreleasePool *pool;
   int i, c;
   NGLdapEntry *entry;
@@ -2049,7 +2113,6 @@ _makeLDAPChanges (NGLdapConnection *ldapConnection,
       members = [NSMutableArray new];
       uids = [NSMutableArray array];
       dns = [NSMutableArray array];
-      logins = [NSMutableArray array];
 
       // We check if it's a static group
       // Fetch "members" - we get DNs
@@ -2084,8 +2147,16 @@ _makeLDAPChanges (NGLdapConnection *ldapConnection,
               user = [SOGoUser userWithLogin: login  roles: nil];
               if (user)
                 {
-                  [logins addObject: login];
-                  [members addObject: user];
+                  contactInfos = [self lookupContactEntryWithUIDorEmail: login inDomain: nil];
+                  if ([contactInfos objectForKey: @"isGroup"])
+                    {
+                      subusers = [self membersForGroupWithUID: login];
+                      [members addObjectsFromArray: subusers];
+                    }
+                  else
+                    {
+                      [members addObject: user];
+                    }
                 }
               [pool release];
             }
@@ -2101,23 +2172,22 @@ _makeLDAPChanges (NGLdapConnection *ldapConnection,
                   contactInfos = [self lookupContactEntryWithUIDorEmail: login inDomain: nil];
                   if ([contactInfos objectForKey: @"isGroup"])
                     {
-                      users = [self membersForGroupWithUID: login];
-                      [members addObjectsFromArray: users];
+                      subusers = [self membersForGroupWithUID: login];
+                      [members addObjectsFromArray: subusers];
                     }
                   else
                     {
-                      [logins addObject: login];
                       [members addObject: user];
                     }
                 }
               [pool release];
             }
 
-
           // We are done fetching members, let's cache the members of the group
           // (ie., their UIDs) in memcached to speed up -groupWithUIDHasMemberWithUID.
+          logins = [members resultsOfSelector: @selector (loginInDomain)];
           [[SOGoCache sharedCache] setValue: [logins componentsJoinedByString: @","]
-            forKey: [NSString stringWithFormat: @"%@+%@", uid, _domain]];
+                                     forKey: [NSString stringWithFormat: @"%@+%@", uid, _domain]];
         }
       else
         {
