@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2006-2020 Inverse inc.
+  Copyright (C) 2006-2021 Inverse inc.
 
   This file is part of SOGo.
 
@@ -884,6 +884,9 @@
   [mailAccount setObject: identities forKey: @"identities"];
   [identities release];
 
+  if ([_defaults mailForceDefaultIdentity])
+    [mailAccount setObject: [NSNumber numberWithBool: YES] forKey: @"forceDefaultIdentity"];
+
   // 6. Receipts
   if ([_defaults allowUserReceipt])
     {
@@ -942,7 +945,7 @@
   [mailAccount setObject: delegates  forKey: @"delegates"];
 
   // 9. Security
-  if ([[_defaults mailCertificate] length])
+  if ([[self domainDefaults] mailCertificateEnabled] && [[_defaults mailCertificate] length])
     {
       security = [NSMutableDictionary new];
 
@@ -1126,7 +1129,7 @@
   return [authValue boolValue];
 }
 
-- (NSString *) googleAuthenticatorKey
+- (NSString *) totpKey
 {
 #if defined(MFA_CONFIG)
   NSString *key, *result;
@@ -1135,7 +1138,7 @@
 
   size_t s_len, secret_len;
 
-  key = [[[self userSettings] userSalt] substringToIndex: 12];
+  key = [[[self userSettings] userPrivateSalt] substringToIndex: 12];
   s = [key UTF8String];
   s_len = strlen(s);
 
