@@ -1,6 +1,6 @@
 /* UIxContactFoldersView.m - this file is part of SOGo
  *
- * Copyright (C) 2006-2016 Inverse inc.
+ * Copyright (C) 2006-2021 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 
 
 #import <SOGo/SOGoPermissions.h>
+#import <SOGo/SOGoSource.h>
 #import <SOGo/SOGoSystemDefaults.h>
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserSettings.h>
@@ -153,8 +154,8 @@ Class SOGoContactSourceFolderK, SOGoGCSFolderK;
   NSMutableArray *sortedFolders;
   NSMutableDictionary *contact, *uniqueContacts;
   unsigned int i, j, max;
-  NSSortDescriptor *commonNameDescriptor;
-  BOOL excludeGroups, excludeLists;
+  NSSortDescriptor *containerTypeDescriptor, *commonNameDescriptor;
+  BOOL excludeGroups, excludeLists, priorityGcs;
 
   searchText = [self queryParameterForKey: @"search"];
   if ([searchText length] >= [self minimumSearchLength])
@@ -162,6 +163,7 @@ Class SOGoContactSourceFolderK, SOGoGCSFolderK;
       // NSLog(@"Search all contacts: %@", searchText);
       excludeGroups = [[self queryParameterForKey: @"excludeGroups"] boolValue];
       excludeLists = [[self queryParameterForKey: @"excludeLists"] boolValue];
+      priorityGcs = [[self queryParameterForKey: @"priority"] isEqualToString: @"gcs"];
       domain = [[context activeUser] domain];
       folders = nil;
       NS_DURING
@@ -182,11 +184,13 @@ Class SOGoContactSourceFolderK, SOGoGCSFolderK;
         {
           folder = [folders objectAtIndex: i];
 	  /* We first search in LDAP folders (in case of duplicated entries in GCS folders) */
-          if ([folder isKindOfClass: SOGoContactSourceFolderK])
+          if (([folder isKindOfClass: SOGoContactSourceFolderK] && !priorityGcs) ||
+              ([folder isKindOfClass: SOGoGCSFolderK] && priorityGcs))
             [sortedFolders insertObject: folder atIndex: 0];
           else
             [sortedFolders addObject: folder];
         }
+      NSLog(@"sortedFolders = %@", sortedFolders);
       for (i = 0; i < max; i++)
         {
           folder = [sortedFolders objectAtIndex: i];
@@ -201,6 +205,16 @@ Class SOGoContactSourceFolderK, SOGoGCSFolderK;
               contact = [contacts objectAtIndex: j];
               [contact setObject: [folder displayName]
                           forKey: @"containerName"];
+              if (priorityGcs)
+                {
+                  // Add the container type for sorting
+                  if ([folder isKindOfClass: SOGoContactSourceFolderK])
+                    [contact setObject: @"source"
+                                forKey: @"containerType"];
+                  else
+                    [contact setObject: @"gcs"
+                                forKey: @"containerType"];
+                }
               //NSLog(@"   found %@ (%@) ? %@", [contact objectForKey: @"c_name"], mail,
               //      [contact description]);
               if (!excludeLists && [[contact objectForKey: @"c_component"]
@@ -226,10 +240,13 @@ Class SOGoContactSourceFolderK, SOGoGCSFolderK;
         }
       if ([uniqueContacts count] > 0)
         {
+          // Optionally return contacts from GCS address books first
+          containerTypeDescriptor = [[NSSortDescriptor alloc] initWithKey: @"containerType"
+                                                                ascending: priorityGcs?YES:NO];
           // Sort the contacts by display name
           commonNameDescriptor = [[NSSortDescriptor alloc] initWithKey: @"c_cn"
-                                                             ascending:YES];
-          descriptors = [NSArray arrayWithObjects: commonNameDescriptor, nil];
+                                                             ascending: YES];
+          descriptors = [NSArray arrayWithObjects: containerTypeDescriptor, commonNameDescriptor, nil];
           [commonNameDescriptor release];
           sortedContacts = [[uniqueContacts allValues]
                              sortedArrayUsingDescriptors: descriptors];
@@ -283,7 +300,7 @@ Class SOGoContactSourceFolderK, SOGoGCSFolderK;
   NSString *userLogin, *owner;
   NSArray *folders, *allACLs;
   NSDictionary *folderAttrs;
-  id <SOGoContactFolder> currentFolder;
+  SOGoFolder <SOGoContactFolder> *currentFolder;
 
   BOOL objectCreator, objectEditor, objectEraser, synchronize;
   int max, i;
@@ -305,11 +322,11 @@ Class SOGoContactSourceFolderK, SOGoGCSFolderK;
       if ([currentFolder respondsToSelector: @selector(cardDavURL)])
         {
           urls = [NSMutableDictionary dictionaryWithObjectsAndKeys:
-                                        [currentFolder cardDavURL], @"cardDavURL",
+                                        [(SOGoContactGCSFolder *)currentFolder cardDavURL], @"cardDavURL",
                                       nil];
           if ([self isPublicAccessEnabled])
             {
-              [urls setObject: [currentFolder publicCardDavURL] forKey: @"publicCardDavURL"];
+              [urls setObject: [(SOGoContactGCSFolder *)currentFolder publicCardDavURL] forKey: @"publicCardDavURL"];
             }
         }
 
@@ -323,7 +340,7 @@ Class SOGoContactSourceFolderK, SOGoGCSFolderK;
                                [NSNumber numberWithBool: objectEraser], @"objectEraser", nil];
 
       if ([currentFolder isKindOfClass: SOGoGCSFolderK])
-        synchronize = [currentFolder synchronize];
+        synchronize = [(SOGoGCSFolder *)currentFolder synchronize];
       else
         synchronize = NO;
 
@@ -339,12 +356,12 @@ Class SOGoContactSourceFolderK, SOGoGCSFolderK;
                                   owner, @"owner",
                                   [NSNumber numberWithBool: [currentFolder isKindOfClass: SOGoGCSFolderK]], @"isEditable",
                                   [NSNumber numberWithBool: [currentFolder isKindOfClass: SOGoContactSourceFolderK]
-                                            && ![currentFolder isPersonalSource]], @"isRemote",
+                                            && ![(SOGoContactSourceFolder *)currentFolder isPersonalSource]], @"isRemote",
                                   [NSNumber numberWithBool: [currentFolder isKindOfClass: SOGoContactSourceFolderK]
-                                            && [currentFolder listRequiresDot]], @"listRequiresDot",
+                                            && [(SOGoContactSourceFolder *)currentFolder listRequiresDot]], @"listRequiresDot",
                                   acls, @"acls",
                                   urls, @"urls",
-                                  [currentFolder searchFields], @"searchFields",
+                                  [(<SOGoSource>)currentFolder searchFields], @"searchFields",
                                   nil];
       [foldersAttrs addObject: folderAttrs];
     }
