@@ -18,8 +18,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSData.h>
 #import <Foundation/NSDictionary.h>
+#import <Foundation/NSData.h>
 
 #import <SaxObjC/SaxLexicalHandler.h>
 #import <SaxObjC/SaxXMLReaderFactory.h>
@@ -113,7 +113,7 @@ _xmlCharsetForCharset (NSString *charset)
   NSMutableString *result;
   NSMutableString *css;
   NSDictionary *attachmentIds;
-  int ignoredContent;
+  int ignoredContent, embeddedCSSLevel;
   NSString *ignoreTag;
   BOOL inBody;
   BOOL inStyle;
@@ -210,6 +210,7 @@ _xmlCharsetForCharset (NSString *charset)
   inStyle = NO;
   inCSSDeclaration = NO;
   hasEmbeddedCSS = NO;
+  embeddedCSSLevel = 0;
 }
 
 - (void) endDocument
@@ -347,21 +348,12 @@ _xmlCharsetForCharset (NSString *charset)
         {
           if (*currentChar == '}')
             {
+              // Prefix CSS rule including ending curly bracket
               inCSSDeclaration = NO;
-              if (hasEmbeddedCSS)
-                {
-                  // End of at-rule definition; remove it from the stylesheet
-                  hasEmbeddedCSS = NO;
-                  start = currentChar + 1;
-                }
-              else
-                {
-                  // Prefix CSS rule including ending curly bracket
-                  length = (currentChar - start) + 1;
-                  [declaration appendString: [NSString stringWithCharacters: start length: length]];
-                  [css appendString: declaration];
-                  start = currentChar + 1;
-                }
+              length = (currentChar - start) + 1;
+              [declaration appendString: [NSString stringWithCharacters: start length: length]];
+              [css appendString: declaration];
+              start = currentChar + 1;
             }
           else if (*currentChar == ';')
             {
@@ -389,53 +381,63 @@ _xmlCharsetForCharset (NSString *charset)
         {
           if (*currentChar == '{')
             {
-              // Start of rule declaration
-              inCSSDeclaration = YES;
-              if (!hasEmbeddedCSS)
+              if (hasEmbeddedCSS)
                 {
+                  embeddedCSSLevel++;
+                }
+              else
+                {
+                  // Start of rule declaration
+                  inCSSDeclaration = YES;
                   length = (currentChar - start);
                   [rule appendFormat: @".SOGoHTMLMail-CSS-Delimiter %@ {",
                         [NSString stringWithCharacters: start length: length]];
                   [css appendString: rule];
+                  rule = [NSMutableString string];
+                  declaration = [NSMutableString string];
                 }
-              rule = [NSMutableString string];
-              declaration = [NSMutableString string];
               start = currentChar + 1;
             }
           if (*currentChar == '}')
             {
-              // CSS syntax error: ending declaration character while not in a CSS declaration.
-              // Ignore eveything from last CSS declaration.
-              start = currentChar + 1;
-              rule = [NSMutableString string];
-            }
-          else if (hasEmbeddedCSS)
-            {
-              if (*currentChar == ';')
+              if (hasEmbeddedCSS)
                 {
-                  // End of at-rule definition; remove it from the stylesheet
-                  hasEmbeddedCSS = NO;
-                  start = currentChar + 1;
+                  embeddedCSSLevel--;
+                  if (embeddedCSSLevel <= 0)
+                    hasEmbeddedCSS = NO;
                 }
+              else
+                {
+                  // CSS syntax error: ending declaration character while not in a CSS declaration.
+                  // Ignore eveything from last CSS declaration.
+                  rule = [NSMutableString string];
+                }
+              start = currentChar + 1;
             }
           else if (*currentChar == ',')
             {
-              // Prefix CSS selector
-              length = (currentChar - start);
-              [rule appendFormat: @" .SOGoHTMLMail-CSS-Delimiter %@,",
-                    [NSString stringWithCharacters: start length: length]];
+              if (!hasEmbeddedCSS)
+                {
+                  // Prefix CSS selector
+                  length = (currentChar - start);
+                  [rule appendFormat: @" .SOGoHTMLMail-CSS-Delimiter %@,",
+                        [NSString stringWithCharacters: start length: length]];
+                }
               start = currentChar + 1;
             }
           else if (*currentChar == '@')
             {
               // Start of at-rule definition
               hasEmbeddedCSS = YES;
+              embeddedCSSLevel = 0;
             }
         }
     }
   if (currentChar > start)
-    [css appendString: [NSString stringWithCharacters: start
-                                               length: (currentChar - start)]];
+    {
+      [css appendString: [NSString stringWithCharacters: start
+                                                 length: (currentChar - start)]];
+    }
 }
 
 - (void) startElement: (NSString *) _localName
@@ -485,9 +487,7 @@ _xmlCharsetForCharset (NSString *charset)
             {
               skipAttribute = NO;
               name = [[_attributes nameAtIndex: count] lowercaseString];
-              if ([name hasPrefix: @"ON"])
-                skipAttribute = YES;
-              else if ([name isEqualToString: @"src"])
+              if ([name isEqualToString: @"src"])
                 {
                   value = [_attributes valueAtIndex: count];
                   if ([value hasPrefix: @"cid:"])
@@ -506,22 +506,23 @@ _xmlCharsetForCharset (NSString *charset)
                   else
                     skipAttribute = YES;
                 }
-              else if (([name isEqualToString: @"background"] ||
-                       ([name isEqualToString: @"data"]
-                        || [name isEqualToString: @"classid"]))
-                       && [lowerName isEqualToString: @"object"])
+              else if ([name isEqualToString: @"background"] ||
+                       (([name isEqualToString: @"data"]
+                         || [name isEqualToString: @"classid"])
+                        && [lowerName isEqualToString: @"object"]))
                 {
                   value = [_attributes valueAtIndex: count];
                   name = [NSString stringWithFormat: @"unsafe-%@", name];
                 }
               else if ([name isEqualToString: @"href"]
-                       || [name isEqualToString: @"action"])
+                       || [name isEqualToString: @"action"]
+                       || [name isEqualToString: @"formaction"])
                 {
-                  value = [_attributes valueAtIndex: count];
-                  skipAttribute = ([value rangeOfString: @"://"].location
-                                   == NSNotFound
+                  value = [[_attributes valueAtIndex: count] lowercaseString];
+                  skipAttribute = ([value rangeOfString: @"://"].location == NSNotFound
                                    && ![value hasPrefix: @"mailto:"]
-                                   && ![value hasPrefix: @"#"]);
+                                   && ![value hasPrefix: @"#"]) ||
+                    [value hasPrefix: @"javascript:"];
                   if (!skipAttribute)
                     [resultPart appendString: @" rel=\"noopener\""];
                 }
@@ -532,38 +533,9 @@ _xmlCharsetForCharset (NSString *charset)
                   if ([value rangeOfString: @"url" options: NSCaseInsensitiveSearch].location != NSNotFound)
                     name = [NSString stringWithFormat: @"unsafe-%@", name];
                 }
-	      else if (
-		       // Mouse Events
-		       [name isEqualToString: @"onclick"] ||
-		       [name isEqualToString: @"ondblclick"] ||
-		       [name isEqualToString: @"onmousedown"] ||
-		       [name isEqualToString: @"onmousemove"] ||
-		       [name isEqualToString: @"onmouseout"] ||
-		       [name isEqualToString: @"onmouseup"] ||
-		       [name isEqualToString: @"onmouseover"] ||
-                       [name isEqualToString: @"onpointerrawupdate"] ||
-
-		       // Keyboard Events
-		       [name isEqualToString: @"onkeydown"] ||
-		       [name isEqualToString: @"onkeypress"] ||
-		       [name isEqualToString: @"onkeyup"] ||
-
-		       // Frame/Object Events
-		       [name isEqualToString: @"onabort"] ||
-		       [name isEqualToString: @"onerror"] ||
-		       [name isEqualToString: @"onload"] ||
-		       [name isEqualToString: @"onresize"] ||
-		       [name isEqualToString: @"onscroll"]  ||
-		       [name isEqualToString: @"onunload"] ||
-
-		       // Form Events
-		       [name isEqualToString: @"onblur"] ||
-		       [name isEqualToString: @"onchange"] ||
-		       [name isEqualToString: @"onfocus"] ||
-		       [name isEqualToString: @"onreset"] ||
-		       [name isEqualToString: @"onselect"] ||
-		       [name isEqualToString: @"onsubmit"]) 
+	      else if ([name hasPrefix: @"on"])
 		{
+                  // on Events
 		  skipAttribute = YES;
 		}
               else
