@@ -1,6 +1,6 @@
 /* UIxMailPartSignedViewer.m - this file is part of SOGo
  *
- * Copyright (C) 2009-2018 Inverse inc.
+ * Copyright (C) 2009-2021 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,8 +29,12 @@
 #import <Foundation/NSDictionary.h>
 #import <Foundation/NSValue.h>
 
+#import <NGExtensions/NSObject+Logs.h>
+
+#import <NGImap4/NGImap4EnvelopeAddress.h>
 #import <NGMime/NGMimeMultipartBody.h>
 
+#import <SOGo/NSDictionary+Utilities.h>
 #import <SOGo/NSString+Utilities.h>
 
 #import <Mailer/SOGoMailObject.h>
@@ -173,15 +177,18 @@
 	  ERR_load_crypto_strings();
           SSL_load_error_strings();
           sslError = ERR_reason_error_string(err);
-          validationMessage = [[self labelForKey: [NSString stringWithUTF8String: sslError ? sslError : @"No error information available"]] retain];
+          validationMessage = [[self labelForKey: sslError ? [NSString stringWithUTF8String: sslError] : @"Digital signature is not valid"] retain];
 #elif OPENSSL_VERSION_NUMBER < 0x10100000L
           const char* sslError;
 	  ERR_load_crypto_strings();
           SSL_load_error_strings();
           sslError = ERR_reason_error_string(err);
-          validationMessage = [[self labelForKey: [NSString stringWithUTF8String: sslError ? sslError : @"No error information available"]] retain];
+          validationMessage = [[self labelForKey: sslError ? [NSString stringWithUTF8String: sslError] : @"Digital signature is not valid"] retain];
 #else
-	  validationMessage = [[self labelForKey: @"No error information available"] retain];
+          const char* sslError;
+          ERR_load_ERR_strings();
+          sslError = ERR_reason_error_string(err);
+          validationMessage = [[self labelForKey: sslError ? [NSString stringWithUTF8String: sslError] : @"Digital signature is not valid"] retain];
 #endif /* HAVE_GNUTLS */
       }
     }
@@ -190,11 +197,56 @@
   BIO_free (msgBio);
   if (inData)
     BIO_free (inData);
+
   
   if (validSignature)
-    validationMessage = [NSString stringWithString: [self labelForKey: @"Message is signed"]];
+    {
+      BOOL hasMatchingAddress;
+      NSArray *pair, *attributes;
+      NSDictionary *certificate, *values;
+      NSEnumerator *certificatesList, *subjectList;
+      NSString *senderAddress, *label, *value;
+
+      // See https://datatracker.ietf.org/doc/html/rfc8550#section-3
+      // See https://datatracker.ietf.org/doc/html/rfc8550#section-4.4.3
+      // TODO: handle multiple email addresses in SubjectAltName
+      attributes = [NSArray arrayWithObjects: @"commonname", @"subjectaltname", @"emailaddress", nil];
+      validationMessage = [self labelForKey: @"Message is signed"];
+      hasMatchingAddress = NO;
+      value = nil;
+      senderAddress = [[[[[self clientObject] fromEnvelopeAddresses] lastObject] baseEMail] lowercaseString];
+      certificatesList = [certificates objectEnumerator];
+      while ((certificate = [certificatesList nextObject]) && !hasMatchingAddress)
+        {
+          subjectList = [[certificate objectForKey: @"subject"] objectEnumerator];
+          while ((pair = [subjectList nextObject]) && !hasMatchingAddress)
+            {
+              label = [[pair objectAtIndex: 0] lowercaseString];
+              value = [[pair objectAtIndex: 1] lowercaseString];
+              if ([attributes containsObject: label] && [value isEqualToString: senderAddress])
+                {
+                  hasMatchingAddress = 1;
+                }
+            }
+        }
+
+      if (!hasMatchingAddress)
+        {
+          if (value)
+            {
+              values = [NSDictionary dictionaryWithObjectsAndKeys: value, @"certificateCn", nil];
+              validationMessage = [values keysWithFormat: [self labelForKey: @"Message is signed but the certificate (%{certificateCn}) doesn't match the sender email address"]];
+            }
+          else
+            {
+              validationMessage = [self labelForKey: @"Message is signed but the certificate doesn't match the sender email address"];
+            }
+        }
+    }
   else if (!validationMessage)
-    validationMessage = [NSString stringWithString: [self labelForKey: @"Digital signature is not valid"]];
+    {
+      validationMessage = [NSString stringWithString: [self labelForKey: @"Digital signature is not valid"]];
+    }
 
   processed = YES;
 }
