@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2006-2015 Inverse inc.
+  Copyright (C) 2006-2021 Inverse inc.
 
   This file is part of SOGo
 
@@ -26,6 +26,7 @@
   object.
 */
 
+#import <Foundation/NSAutoreleasePool.h>
 #import <Foundation/NSCalendarDate.h>
 #import <Foundation/NSCharacterSet.h>
 #import <Foundation/NSTimeZone.h>
@@ -40,6 +41,7 @@
 #import <NGExtensions/NSNull+misc.h>
 #import <NGExtensions/NSString+misc.h>
 #import <NGExtensions/NSObject+Logs.h>
+#import <NGImap4/NGImap4Connection.h>
 #import <NGImap4/NGImap4Envelope.h>
 
 #import <EOControl/EOQualifier.h>
@@ -231,7 +233,7 @@
   return subject;
 }
 
-- (BOOL) showToAddress 
+- (BOOL) showToAddress
 {
   SOGoMailFolder *co;
 
@@ -419,30 +421,32 @@
 
 - (EOQualifier *) searchQualifier
 {
-  EOQualifier *qualifier, *searchQualifier;
+  EOQualifier *qualifier, *notDeleted, *searchQualifier;
   WORequest *request;
   NSDictionary *sortingAttributes, *content, *filter;
-  NSArray *filters;
-  NSString *searchBy, *searchInput, *searchString, *match;
-  NSMutableArray *searchArray;
-  int nbFilters, i;
+  NSArray *filters, *labels;
+  NSString *searchBy, *searchInput, *searchString, *match, *label;
+  NSMutableArray *qualifiers, *searchArray, *labelQualifiers;
+  BOOL unseenOnly, flaggedOnly;
+  int max, i;
   
   request = [context request];
   content = [[request contentAsString] objectFromJSONString];
-  qualifier = nil;
+  notDeleted = [EOQualifier qualifierWithQualifierFormat: @"(not (flags = %@))", @"deleted"];
+  qualifiers = [NSMutableArray arrayWithObject: notDeleted];
   searchString = nil;
   match = nil;
   filters = [content objectForKey: @"filters"];
+  labels = [content objectForKey: @"labels"];
+  unseenOnly = [[content objectForKey: @"unseenOnly"] boolValue];
+  flaggedOnly = [[content objectForKey: @"flaggedOnly"] boolValue];
 
   if (filters)
     {
-      nbFilters = [filters count];
-      if (nbFilters > 0) {
-        searchArray = [NSMutableArray arrayWithCapacity: nbFilters];
-        sortingAttributes = [content objectForKey: @"sortingAttributes"];
-        if (sortingAttributes)
-          match = [sortingAttributes objectForKey: @"match"]; // AND, OR
-        for (i = 0; i < nbFilters; i++)
+      max = [filters count];
+      if (max > 0) {
+        searchArray = [NSMutableArray arrayWithCapacity: max];
+        for (i = 0; i < max; i++)
           {
             filter = [filters objectAtIndex:i];
             searchBy = [filter objectForKey: @"searchBy"];
@@ -463,36 +467,67 @@
                 [self errorWithFormat: @"Missing parameters in search filter: %@", filter];
               }
           }
+        sortingAttributes = [content objectForKey: @"sortingAttributes"];
+        if (sortingAttributes)
+          match = [sortingAttributes objectForKey: @"match"]; // AND, OR
         if ([match isEqualToString: @"OR"])
           qualifier = [[EOOrQualifier alloc] initWithQualifierArray: searchArray];
         else
           qualifier = [[EOAndQualifier alloc] initWithQualifierArray: searchArray];
+        [qualifier autorelease];
+        [qualifiers addObject: qualifier];
       }
     }
-    
+
+  if (unseenOnly)
+    {
+      searchQualifier = [EOQualifier qualifierWithQualifierFormat: @"(not (flags = %@))", @"seen"];
+      [qualifiers addObject: searchQualifier];
+    }
+  if (flaggedOnly)
+    {
+      searchQualifier = [EOQualifier qualifierWithQualifierFormat: @"(flags = %@)", @"flagged"];
+      [qualifiers addObject: searchQualifier];
+    }
+  if (labels)
+    {
+      max = [labels count];
+      if (max > 0)
+        {
+          labelQualifiers = [NSMutableArray arrayWithCapacity: max];
+            for (i = 0; i < max; i++)
+            {
+              label = [labels objectAtIndex: i];
+              qualifier = [EOQualifier qualifierWithQualifierFormat: @"(flags = %@)", label];
+              [labelQualifiers addObject: qualifier];
+            }
+            if (max > 1)
+              {
+                qualifier = [[EOOrQualifier alloc] initWithQualifierArray: labelQualifiers];
+                [qualifier autorelease];
+              }
+            [qualifiers addObject: qualifier];
+        }
+    }
+
+  if ([qualifiers count] > 1)
+    {
+      qualifier = [[EOAndQualifier alloc] initWithQualifierArray: qualifiers];
+      [qualifier autorelease];
+    }
+  else
+    qualifier = notDeleted;
+
   return qualifier;
 }
 
 - (NSArray *) getSortedUIDsInFolder: (SOGoMailFolder *) mailFolder
 {
-  EOQualifier *qualifier, *fetchQualifier, *notDeleted;
-
   if (!sortedUIDs)
     {
-      notDeleted = [EOQualifier qualifierWithQualifierFormat: @"(not (flags = %@))", @"deleted"];
-      qualifier = [self searchQualifier];
-      if (qualifier)
-        {
-          fetchQualifier = [[EOAndQualifier alloc] initWithQualifiers: notDeleted, qualifier, nil];
-          [fetchQualifier autorelease];
-        }
-      else
-        fetchQualifier = notDeleted;
-    
-      sortedUIDs = [mailFolder fetchUIDsMatchingQualifier: fetchQualifier
+      sortedUIDs = [mailFolder fetchUIDsMatchingQualifier: [self searchQualifier]
                                              sortOrdering: [self imap4SortOrdering]
                                                  threaded: sortByThread];
-    
       [sortedUIDs retain];
     }
 
@@ -647,7 +682,7 @@
   int count;
 
   data = [NSMutableDictionary dictionary];
-  
+
   // TODO: we might want to flush the caches?
   //[folder flushMailCaches];
   [folder expungeLastMarkedFolder];
@@ -660,6 +695,16 @@
       return nil;
     }
   
+  // We first make sure QRESYNC is enabled
+  if (![[folder imap4Connection] enableExtensions: [NSArray arrayWithObject: @"QRESYNC"]])
+    {
+      NSString *tag = [folder davCollectionTag];
+      if (![tag isEqualToString: @"-1"])
+        {
+          [data setObject: tag forKey: @"syncToken"];
+        }
+    }
+
   // Get rid of the extra parenthesis
    // uids = [[[[uids stringValue] stringByReplacingOccurrencesOfString:@"(" withString:@""] stringByReplacingOccurrencesOfString:@")" withString:@""] componentsSeparatedByString:@","];
 
@@ -725,6 +770,7 @@
  * @apiParam {String} filters.searchBy                       Field criteria. Either subject, from, to, cc, or body.
  * @apiParam {String} filters.searchInput                    String to match.
  * @apiParam {String} [filters.negative]                     Reverse the condition when true. Defaults to false.
+ * @apiParam {Boolean} [unseenOnly]                          Filter out seen messages when true. Defaults to false.
  *
  * @apiSuccess (Success 200) {Number} threaded               1 if threading is enabled for the user.
  * @apiSuccess (Success 200) {Number} unseenCount            Number of unread messages
@@ -785,114 +831,197 @@
   return response;
 }
 
+- (id <WOActionResults>) getChangesAction
+{
+  NSArray *changedMessages, *headers;
+  NSDictionary *requestContent, *data, *changedMessage;
+  NSMutableArray *changedUids, *deletedUids;
+  NSString *syncToken, *newSyncToken, *uid;
+  SOGoMailFolder *folder;
+  WORequest *request;
+  WOResponse *response;
+  int i, max;
+
+  request = [context request];
+  requestContent = [[request contentAsString] objectFromJSONString];
+  response = nil;
+  folder = [self clientObject];
+  syncToken = [requestContent objectForKey: @"syncToken"];
+  newSyncToken = [folder davCollectionTag];
+
+  if ([syncToken length] && ![syncToken isEqual: newSyncToken])
+    {
+      // Fetch list of changed uids
+      changedMessages = [folder syncTokenFieldsWithProperties: nil
+                                            matchingSyncToken: syncToken
+                                                     fromDate: nil
+                                                  initialLoad: NO
+                                                 sortOrdering: [self imap4SortOrdering]
+                                                     threaded: sortByThread];
+      if ((max = [changedMessages count]))
+        {
+          // Split new or modified uids from deleted uids
+          changedUids = [NSMutableArray array];
+          deletedUids = [NSMutableArray array];
+          for (i = 0; i < max; i++)
+            {
+              changedMessage = [changedMessages objectAtIndex: i];
+              uid = [[changedMessage allKeys] lastObject];
+              if ([[changedMessage objectForKey: uid] isEqual: [NSNull null]])
+                [deletedUids addObject: uid];
+              else
+                [changedUids addObject: uid];
+            }
+
+          // Fetch headers for new or modified messages
+          headers = [self getHeadersForUIDs: changedUids
+                                   inFolder: folder];
+
+          data = [NSDictionary dictionaryWithObjectsAndKeys:
+                                 changedUids, @"changed",
+                               deletedUids, @"deleted",
+                               headers, @"headers",
+                               newSyncToken, @"syncToken",
+                               [NSNumber numberWithUnsignedInt: [folder unseenCount]], @"unseenCount",
+                               nil];
+          response = [self responseWithStatus: 200 andJSONRepresentation: data];
+        }
+    }
+  if (!response)
+    {
+      data = [NSDictionary dictionaryWithObjectsAndKeys:
+                           newSyncToken, @"syncToken",
+                           nil];
+      response = [self responseWithStatus: 200 andJSONRepresentation: data];
+    }
+
+  return response;
+}
+
 - (NSArray *) getHeadersForUIDs: (NSArray *) uids
 		       inFolder: (SOGoMailFolder *) mailFolder
 {
   UIxEnvelopeAddressFormatter *addressFormatter;
+  NSAutoreleasePool *pool;
   NSMutableArray *headers, *msg, *tags;
   NSEnumerator *msgsList;
   NSArray *to, *from;
   NSDictionary *msgs;
   NSString *msgDate;
-  
-  headers = [NSMutableArray arrayWithCapacity: [uids count]];
-  addressFormatter = [context mailEnvelopeAddressFormatter];
-  
-  // Fetch headers
-  msgs = (NSDictionary *)[mailFolder fetchUIDs: uids
-					 parts: [self fetchKeys]];
+  int count;
 
-  msgsList = [[msgs objectForKey: @"fetch"] objectEnumerator];
-  [self setMessage: [msgsList nextObject]];
+  count = [uids count];
+  headers = [NSMutableArray arrayWithCapacity: count];
 
-  msg = [NSMutableArray arrayWithObjects: @"To", @"hasAttachment", @"isFlagged", @"Subject", @"From", @"isRead", @"Priority", @"RelativeDate", @"Size", @"Flags", @"uid", @"isAnswered", @"isForwarded", nil];
-  [headers addObject: msg];
-  while (message)
+  if (count > 0)
     {
-      // We must check for "umimportant" untagged responses.
-      //
-      // It's generally caused by IMAP server processes sending untagged IMAP responses to SOGo in differnent IMAP
-      // connections (SOGo might use 2-3 per user). Say you ask your messages:
-      //
-      // 127.000.000.001.40725-127.000.000.001.00143: 59 uid fetch 62 (UID FLAGS ENVELOPE RFC822.SIZE BODYSTRUCTURE BODY.PEEK[HEADER.FIELDS (X-PRIORITY)])
-      // 127.000.000.001.00143-127.000.000.001.40725: * 62 FETCH (UID 62 FLAGS (\Seen) RFC822.SIZE 854 ENVELOPE  .... (
-      // * 61 FETCH (FLAGS (\Deleted \Seen))
-      // * 62 FETCH (FLAGS (\Deleted \Seen))
-      // * 63 FETCH (FLAGS (\Deleted \Seen))
-      // 59 OK Fetch completed.
-      //
-      // We must ignore the * 61 .. * 63 untagged responses.
-      //
-      if (![message objectForKey: @"uid"])
-	{
-	  [self setMessage: [msgsList nextObject]];
-	  continue;
-	}
+      addressFormatter = [context mailEnvelopeAddressFormatter];
 
-      msg = [NSMutableArray arrayWithCapacity: 12];
+      // Fetch headers
+      msgs = (NSDictionary *)[mailFolder fetchUIDs: uids
+                                             parts: [self fetchKeys]];
 
-      // Columns data
-
-      // To
-      to = [[message objectForKey: @"envelope"] to];
-      if ([to count] > 0)
-	[msg addObject: [addressFormatter dictionariesForArray: to]];
-      else
-	[msg addObject: @""];
-
-      // hasAttachment
-      [msg addObject: [NSNumber numberWithBool: [self hasMessageAttachment]]];
-
-      // isFlagged
-      [msg addObject: [NSNumber numberWithBool: [self isMessageFlagged]]];
-
-      // Subject
-      [msg addObject: [self messageSubject]];
-      
-      // From
-      from = [[message objectForKey: @"envelope"] from];
-      if ([from count] > 0)
-	[msg addObject: [addressFormatter dictionariesForArray: from]];
-      else
-	[msg addObject: @""];
-      
-      // isRead
-      [msg addObject: [NSNumber numberWithBool: [self isMessageRead]]];
-      
-      // Priority
-      [msg addObject: [self messagePriority]];
-
-      // Relative Date
-      msgDate = [self messageDate];
-      if (msgDate == nil)
-	msgDate = @"";
-      [msg addObject: msgDate];
-
-      // Size
-      [msg addObject: [[self sizeFormatter] stringForObjectValue: [message objectForKey: @"size"]]];
-
-      // Mail labels / tags
-      tags = [NSMutableArray arrayWithArray: [message objectForKey: @"flags"]];
-      [tags removeObject: @"answered"];
-      [tags removeObject: @"deleted"];
-      [tags removeObject: @"draft"];
-      [tags removeObject: @"flagged"];
-      [tags removeObject: @"recent"];
-      [tags removeObject: @"seen"];
-      [tags removeObject: @"$forwarded"];
-      [msg addObject: tags];
-
-      // UID
-      [msg addObject: [message objectForKey: @"uid"]];
-      [headers addObject: msg];
-
-      // isAnswered
-      [msg addObject: [NSNumber numberWithBool: [self isMessageAnswered]]];
-
-      // isForwarded
-      [msg addObject: [NSNumber numberWithBool: [self isMessageForwarded]]];
-      
+      msgsList = [[msgs objectForKey: @"fetch"] objectEnumerator];
       [self setMessage: [msgsList nextObject]];
+
+      msg = [NSMutableArray arrayWithObjects: @"To", @"hasAttachment", @"isFlagged", @"Subject", @"From", @"isRead", @"Priority", @"RelativeDate", @"Size", @"Flags", @"uid", @"isAnswered", @"isForwarded", nil];
+      [headers addObject: msg];
+      count = 0;
+      pool = [[NSAutoreleasePool alloc] init];
+      while (message)
+        {
+          // We must check for "umimportant" untagged responses.
+          //
+          // It's generally caused by IMAP server processes sending untagged IMAP responses to SOGo in differnent IMAP
+          // connections (SOGo might use 2-3 per user). Say you ask your messages:
+          //
+          // 127.000.000.001.40725-127.000.000.001.00143: 59 uid fetch 62 (UID FLAGS ENVELOPE RFC822.SIZE BODYSTRUCTURE BODY.PEEK[HEADER.FIELDS (X-PRIORITY)])
+          // 127.000.000.001.00143-127.000.000.001.40725: * 62 FETCH (UID 62 FLAGS (\Seen) RFC822.SIZE 854 ENVELOPE  .... (
+          // * 61 FETCH (FLAGS (\Deleted \Seen))
+          // * 62 FETCH (FLAGS (\Deleted \Seen))
+          // * 63 FETCH (FLAGS (\Deleted \Seen))
+          // 59 OK Fetch completed.
+          //
+          // We must ignore the * 61 .. * 63 untagged responses.
+          //
+          if (![message objectForKey: @"uid"])
+            {
+              [self setMessage: [msgsList nextObject]];
+              continue;
+            }
+
+          msg = [NSMutableArray arrayWithCapacity: 12];
+
+          // Columns data
+
+          // To
+          to = [[message objectForKey: @"envelope"] to];
+          if ([to count] > 0)
+            [msg addObject: [addressFormatter dictionariesForArray: to]];
+          else
+            [msg addObject: @""];
+
+          // hasAttachment
+          [msg addObject: [NSNumber numberWithBool: [self hasMessageAttachment]]];
+
+          // isFlagged
+          [msg addObject: [NSNumber numberWithBool: [self isMessageFlagged]]];
+
+          // Subject
+          [msg addObject: [self messageSubject]];
+      
+          // From
+          from = [[message objectForKey: @"envelope"] from];
+          if ([from count] > 0)
+            [msg addObject: [addressFormatter dictionariesForArray: from]];
+          else
+            [msg addObject: @""];
+      
+          // isRead
+          [msg addObject: [NSNumber numberWithBool: [self isMessageRead]]];
+      
+          // Priority
+          [msg addObject: [self messagePriority]];
+
+          // Relative Date
+          msgDate = [self messageDate];
+          if (msgDate == nil)
+            msgDate = @"";
+          [msg addObject: msgDate];
+
+          // Size
+          [msg addObject: [[self sizeFormatter] stringForObjectValue: [message objectForKey: @"size"]]];
+
+          // Mail labels / tags
+          tags = [NSMutableArray arrayWithArray: [message objectForKey: @"flags"]];
+          [tags removeObject: @"answered"];
+          [tags removeObject: @"deleted"];
+          [tags removeObject: @"draft"];
+          [tags removeObject: @"flagged"];
+          [tags removeObject: @"recent"];
+          [tags removeObject: @"seen"];
+          [tags removeObject: @"$forwarded"];
+          [msg addObject: tags];
+
+          // UID
+          [msg addObject: [message objectForKey: @"uid"]];
+          [headers addObject: msg];
+
+          // isAnswered
+          [msg addObject: [NSNumber numberWithBool: [self isMessageAnswered]]];
+
+          // isForwarded
+          [msg addObject: [NSNumber numberWithBool: [self isMessageForwarded]]];
+      
+          [self setMessage: [msgsList nextObject]];
+
+          count++;
+          if (count % 10 == 0)
+            {
+              [pool release];
+              pool = [[NSAutoreleasePool alloc] init];
+            }
+        }
     }
 
   return headers;
