@@ -1002,7 +1002,7 @@ static NSArray *reminderValues = nil;
   return [NSString stringWithString: SOGoVersion];
 }
 
-- (BOOL) isGoogleAuthenticatorEnabled
+- (BOOL) isTotpEnabled
 {
 #if defined(MFA_CONFIG)
   return YES;
@@ -1011,9 +1011,9 @@ static NSArray *reminderValues = nil;
 #endif
 }
 
-- (NSString *) googleAuthenticatorKey
+- (NSString *) totpKey
 {
-  return [[context activeUser] googleAuthenticatorKey];
+  return [[context activeUser] totpKey];
 }
 
 //
@@ -1309,8 +1309,8 @@ static NSArray *reminderValues = nil;
         [target setObject: identities forKey: @"SOGoMailIdentities"];
       if ([[account objectForKey: @"forceDefaultIdentity"] boolValue])
         [target setObject: [NSNumber numberWithBool: YES] forKey: @"SOGoMailForceDefaultIdentity"];
-      else if ([target objectForKey: @"SOGoMailforceDefaultIdentity"])
-        [target removeObjectForKey: @"SOGoMailforceDefaultIdentity"];
+      else if ([target objectForKey: @"SOGoMailForceDefaultIdentity"])
+        [target removeObjectForKey: @"SOGoMailForceDefaultIdentity"];
       [self _extractMainReceiptsPreferences: [account objectForKey: @"receipts"]  inDictionary: target];
       [self _extractMainSecurityPreferences: [account objectForKey: @"security"]  inDictionary: target];
     }
@@ -1567,6 +1567,14 @@ static NSArray *reminderValues = nil;
 
   if ((v = [o objectForKey: @"settings"]))
     {
+      // Remove ReloadWebCalendars if necessary
+      NSMutableDictionary *calendarModule = [o objectForKey: @"Calendar"];
+      if (calendarModule && [calendarModule objectForKey: @"ReloadWebCalendars"] == nil)
+        {
+          calendarModule = [[user userSettings] objectForKey: @"Calendar"];
+          [calendarModule removeObjectForKey: @"ReloadWebCalendars"];
+        }
+
       [[[user userSettings] source] setValues: v];
       [[user userSettings] synchronize];
     }
