@@ -102,6 +102,61 @@
     return collection;
   };
 
+  /**
+   * @memberof Account
+   * @desc Refresh the unseen count for all required mailboxes.
+   *       Starts a timer if user choose to automatically refresh folders.
+   * @param {array} [string] - the paths of the folders
+   */
+  Account.refreshUnseenCount = function(folders) {
+    var unseenCountFolders,
+        fetchAllUnseenCountFolders = (Account.$Preferences.defaults.SOGoMailFetchAllUnseenCountFolders === 1),
+        refreshViewCheck = Account.$Preferences.defaults.SOGoRefreshViewCheck;
+
+    if (fetchAllUnseenCountFolders)
+      unseenCountFolders = [];
+    else if (folders)
+      unseenCountFolders = folders;
+    else
+      throw Error('SOGoMailFetchAllUnseenCountFolders is disabled and no folders list provided');
+
+    _.forEach(Account.$accounts, function(account) {
+      if (fetchAllUnseenCountFolders) {
+        // Include all mailboxes
+        _.forEach(account.$$flattenMailboxes, function(mailbox) {
+          unseenCountFolders.push(mailbox.id);
+        });
+      }
+      else {
+        // Always include the INBOX
+        if (!_.includes(unseenCountFolders, account.id + '/folderINBOX'))
+          unseenCountFolders.push(account.id + '/folderINBOX');
+
+        _.forEach(account.$$flattenMailboxes, function(mailbox) {
+          if (angular.isDefined(mailbox.unseenCount) &&
+              !_.includes(unseenCountFolders, mailbox.id))
+            unseenCountFolders.push(mailbox.id);
+        });
+      }
+    });
+
+    Account.$$resource.post('', 'unseenCount', {mailboxes: unseenCountFolders}).then(function(data) {
+      _.forEach(Account.$accounts, function(account) {
+        _.forEach(account.$$flattenMailboxes, function(mailbox) {
+          if (angular.isDefined(data[mailbox.id])) {
+            mailbox.unseenCount = data[mailbox.id];
+          }
+        });
+      });
+    });
+
+    if (refreshViewCheck && refreshViewCheck != 'manually') {
+      if (Account.$refreshUnseenCount)
+        Account.$timeout.cancel(Account.$refreshUnseenCount);
+      Account.$refreshUnseenCount = Account.$timeout(angular.bind(this, Account.refreshUnseenCount, folders), refreshViewCheck.timeInterval()*1000);
+    }
+  };
+
   /**
    * @function getLength
    * @memberof Account.prototype
@@ -337,6 +392,16 @@
     return identity.textSignature;
   };
 
+  /**
+   * @function $hasCertificate
+   * @memberof Account.prototype
+   * @desc Return true if the user has a S/MIME certificate for this account
+   * @returns a boolean value
+   */
+  Account.prototype.$hasCertificate = function() {
+    return this.security && this.security.hasCertificate;
+  };
+
   /**
    * @function $certificate
    * @memberof Account.prototype
@@ -346,7 +411,7 @@
   Account.prototype.$certificate = function() {
     var _this = this;
 
-    if (this.security && this.security.hasCertificate) {
+    if (this.$hasCertificate()) {
       if (this.$$certificate)
         return Account.$q.when(this.$$certificate);
       else {
@@ -384,9 +449,16 @@
   Account.prototype.updateQuota = function(data) {
     var percent, format, description;
 
-    percent = (Math.round(data.usedSpace * 10000 / data.maxQuota) / 100);
-    format = l("quotasFormat");
-    description = format.formatted(percent, Math.round(data.maxQuota/10.24)/100);
+    if (data.maxQuota) {
+      percent = (Math.round(data.usedSpace * 10000 / data.maxQuota) / 100);
+      format = l("quotasFormat");
+      description = format.formatted(percent, Math.round(data.maxQuota/10.24)/100);
+    }
+    else if (data.maxMessages) {
+      percent = (Math.round(data.messagesCount * 10000 / data.maxMessages) / 100);
+      format = l("messageQuotasFormat");
+      description = format.formatted(percent, data.maxMessages);
+    }
 
     this.$quota = { percent: percent, description: description };
   };
