@@ -26,8 +26,10 @@
         this.init(futureMessageData);
       }
       this.uid = parseInt(futureMessageData.uid);
+      this.selected = !!futureMessageData.selected;
       this.level = parseInt(futureMessageData.level);
       this.first = parseInt(futureMessageData.first) === 1;
+      this.flags = [];
       if (this.first) {
         this.threadCount = parseInt(futureMessageData.count);
         this.collapsed = (futureMessageData.collapsed === true);
@@ -61,6 +63,8 @@
     // Initialize tags form user's defaults
     if (Preferences.defaults.SOGoMailLabelsColors) {
       Message.$tags = Preferences.defaults.SOGoMailLabelsColors;
+    } else {
+      Message.$tags = {};
     }
     if (Preferences.defaults.SOGoMailDisplayRemoteInlineImages &&
         Preferences.defaults.SOGoMailDisplayRemoteInlineImages == 'always') {
@@ -357,9 +361,23 @@
                 };
               }
             }
-            _.forEach(part.content, function(mixedPart) {
-              _visit(mixedPart);
+            var winmail = _.find(part.content, function(mixedPart) {
+              // Ignore empty content -- that could mean a decoding error server-side.
+              return mixedPart.type == 'UIxMailPartTnefViewer' && mixedPart.content.length > 0;
             });
+
+            if (winmail && !_.find(part.content, function(mixedPart) {
+              return mixedPart.type == 'UIxMailPartAlternativeViewer';
+            })) {
+              // If there's no alternate part in the message, show the winmail.dat attachment only.
+              // Otherwise, show all parts.
+              _visit(winmail);
+            }
+            else {
+              _.forEach(part.content, function(mixedPart) {
+                _visit(mixedPart);
+              });
+            }
           }
           else {
             if (angular.isUndefined(part.safeContent)) {
@@ -478,7 +496,18 @@
    * @returns a promise of the HTTP operation
    */
   Message.prototype.addTag = function(tag) {
-    return this.$addOrRemoveTag('add', tag);
+    var _this = this,
+        _tag = tag.replace(/^_\$/, '$');
+    return this.$mailbox.getLabels().then(function(labels) {
+      var reload = !_.find(labels, function(label) {
+        return label.imapName == _tag;
+      });
+      return _this.$addOrRemoveTag('add', tag).then(function() {
+        if (reload)
+          // Update the list of labels for the mailbox
+          _this.$mailbox.getLabels({reload: true});
+      });
+    });
   };
 
   /**
@@ -538,6 +567,22 @@
     return Message.$$resource.post(this.$absolutePath(), 'sendMDN');
   };
 
+  /**
+   * @function hasAttachments
+   * @memberof Message.prototype
+   * @returns true if there's one ore more attached files
+   */
+  Message.prototype.hasAttachments = function(content) {
+    var _this = this;
+
+    return !!_.find(content || this.parts.content, function(part) {
+      if (angular.isArray(part.content)) {
+        return _this.hasAttachments(part.content);
+      }
+      return part.type == 'UIxMailPartLinkViewer' || part.type == 'UIxMailPartImageViewer';
+    });
+  };
+
   /**
    * @function $deleteAttachment
    * @memberof Message.prototype
@@ -590,8 +635,12 @@
       action = 'markMessageUncollapse';
 
     this.collapsed = !this.collapsed;
+    _this.$mailbox.updateVisibleMessages();
 
-    return Message.$$resource.post(this.$absolutePath(), action);
+    return Message.$$resource.post(this.$absolutePath(), action).catch(function() {
+      this.collapsed = !this.collapsed;
+      _this.$mailbox.updateVisibleMessages();
+    });
   };
 
   /**
@@ -729,13 +778,16 @@
             data.encrypt = true;
         }
         if (data.isHTML) {
-          // Sanitize HTML replies to properly display quoted content in CKEditor
-          var html = angular.element('<div>' + data.text + '</div>');
-          html.find('meta').remove();
-          html.find('link').remove();
-          html.find('base').remove();
-          html.find('title').remove();
-          data.text = html.html();
+          // Sanitize HTML replies to properly display quoted content in CKEditor.
+          // Don't use the DOM to avoid triggering any event.
+          var html = data.text;
+          html = html.replace(/<\/?html[^>]*>/g, '');
+          html = html.replace(/<\/?body[^>]*>/g, '');
+          html = html.replace(/<meta[^>]*>.*<\/meta>/g, '');
+          html = html.replace(/<link[^>]*>.*<\/link>/g, '');
+          html = html.replace(/<base[^>]*>.*<\/base>/g, '');
+          html = html.replace(/<title[^>]*>.*<\/title>/g, '');
+          data.text = html;
         }
         angular.extend(message.editable, data);
 
@@ -853,12 +905,12 @@
   };
 
   /**
-   * @function download
+   * @function downloadArchive
    * @memberof Message.prototype
-   * @desc Download the current message
+   * @desc Download the current message as a zip archive
    * @returns a promise of the HTTP operation
    */
-  Message.prototype.download = function() {
+  Message.prototype.downloadArchive = function() {
     var data, options;
 
     data = { uids: [this.uid] };
@@ -868,12 +920,25 @@
   };
 
   /**
-   * @function downloadAttachments
+   * @function download
    * @memberof Message.prototype
-   * @desc Download an archive of all attachments
+   * @desc Download the current message as a eml file
    * @returns a promise of the HTTP operation
    */
-  Message.prototype.downloadAttachments = function() {
+  Message.prototype.download = function() {
+    var options;
+
+    options = { filename: this.subject + '.eml' };
+    return Message.$$resource.download(this.$absolutePath(), 'export', undefined, options);
+  };
+
+  /**
+   * @function downloadAttachments
+   * @memberof Message.prototype
+   * @desc Download a zip archive of all attachments
+   * @returns a promise of the HTTP operation
+   */
+  Message.prototype.downloadAttachmentsArchive = function() {
     var options;
 
     options = { filename: l('attachments') + "-" + this.uid + ".zip" };
