@@ -12,16 +12,17 @@
 
     this.$onInit = function() {
       $scope.isPopup = stateParent.isPopup;
-      this.addRecipient = addRecipient;
+      this.account = stateAccount;
       this.autocomplete = {to: {}, cc: {}, bcc: {}};
       this.autosave = null;
       this.autosaveDrafts = autosaveDrafts;
       this.cancel = cancel;
-      this.contactFilter = contactFilter;
       this.isFullscreen = false;
       this.hideBcc = (stateMessage.editable.bcc.length === 0);
       this.hideCc = (stateMessage.editable.cc.length === 0);
-      this.identities = _.uniq(_.map(stateAccount.identities, 'full'));
+      this.identities = stateAccount.identities;
+      this.fromIdentity = stateMessage.editable.from;
+      this.identitySearchText = '';
       this.message = stateMessage;
       this.recipientSeparatorKeys = [
         $mdConstant.KEY_CODE.ENTER,
@@ -40,9 +41,14 @@
       if (Preferences.defaults.SOGoMailAutoSave)
         // Enable auto-save of draft
         this.autosave = $timeout(this.autosaveDrafts, Preferences.defaults.SOGoMailAutoSave*1000*60);
+
       // Set the locale of CKEditor
       this.localeCode = Preferences.defaults.LocaleCode;
+      this.ckConfig = { language: Preferences.defaults.LocaleCode };
 
+      this.composeType = Preferences.defaults.SOGoMailComposeMessageType;
+
+      this.signaturePlacement = Preferences.defaults.SOGoMailSignaturePlacement;
       this.replyPlacement = Preferences.defaults.SOGoMailReplyPlacement;
       if (this.message.origin && this.message.origin.action == 'forward') {
         // For forwards, place caret at top unconditionally
@@ -55,6 +61,7 @@
       if ($stateParams.actionName == 'reply') {
         stateMessage.$reply().then(function(msgObject) {
           vm.message = msgObject;
+          vm.fromIdentity = msgObject.editable.from;
           vm.hideCc = (!msgObject.editable.cc || msgObject.editable.cc.length === 0);
           vm.hideBcc = (!msgObject.editable.bcc || msgObject.editable.bcc.length === 0);
           _updateFileUploader();
@@ -63,6 +70,7 @@
       else if ($stateParams.actionName == 'replyall') {
         stateMessage.$replyAll().then(function(msgObject) {
           vm.message = msgObject;
+          vm.fromIdentity = msgObject.editable.from;
           vm.hideCc = (!msgObject.editable.cc || msgObject.editable.cc.length === 0);
           vm.hideBcc = (!msgObject.editable.bcc || msgObject.editable.bcc.length === 0);
           _updateFileUploader();
@@ -71,6 +79,7 @@
       else if ($stateParams.actionName == 'forward') {
         stateMessage.$forward().then(function(msgObject) {
           vm.message = msgObject;
+          vm.fromIdentity = msgObject.editable.from;
           _updateFileUploader();
           _addAttachments();
         });
@@ -128,6 +137,7 @@
           vm.message.$setUID(response.uid);
           vm.message.$reload();
           item.inlineUrl = response.lastAttachmentAttrs[0].url;
+          item.file.name = response.lastAttachmentAttrs[0].filename;
           //console.debug(item); console.debug('success = ' + JSON.stringify(response, undefined, 2));
         },
         onCancelItem: function(item, response, status, headers) {
@@ -139,7 +149,7 @@
         onErrorItem: function(item, response, status, headers) {
           $mdToast.show(
             $mdToast.simple()
-              .content(l('Error while uploading the file \"%{0}\":', item.file.name) +
+              .textContent(l('Error while uploading the file \"%{0}\":', item.file.name) +
                        ' ' + (response.message? l(response.message) : ''))
               .position('top right')
               .action(l('OK'))
@@ -213,7 +223,7 @@
         }
         $mdToast.show(
           $mdToast.simple()
-            .content(l('Your email has been saved'))
+            .textContent(l('Your email has been saved'))
             .position('top right')
             .hideDelay(3000));
       });
@@ -244,7 +254,7 @@
         }
         $mdToast.show(
           $mdToast.simple()
-            .content(l('Your email has been sent'))
+            .textContent(l('Your email has been sent'))
             .position('top right')
             .hideDelay(3000));
 
@@ -262,8 +272,8 @@
       vm.isFullscreen = !vm.isFullscreen;
     }
 
-    function contactFilter($query) {
-      return AddressBook.$filterAll($query).then(function(cards) {
+    this.contactFilter = function ($query) {
+      return AddressBook.$filterAll($query, [], {priority: 'gcs'}).then(function(cards) {
         // Divide the matching cards by email addresses so the user can select
         // the recipient address of her choice
         var explodedCards = [];
@@ -277,13 +287,13 @@
           return card.$$fullname + ' ' + card.$$email;
         });
       });
-    }
+    };
 
-    function addRecipient(contact, field) {
+    this.addRecipient = function (contact, field) {
       var recipients, recipient, list, i, address;
       var emailRE = /([\w\!\#$\%\&\'\*\+\-\/\=\?\^\`{\|\}\~]+\.)*[\w\!\#$\%\&\'\*\+\-\/\=\?\^\`{\|\}\~]+@((((([a-z0-9]{1}[a-z0-9\-]{0,62}[a-z0-9]{1})|[a-z])\.)+[a-z]{2,})|(\d{1,3}\.){3}\d{1,3}(\:\d{1,5})?)/i;
 
-      recipients = vm.message.editable[field];
+      recipients = this.message.editable[field];
 
       if (angular.isString(contact)) {
         // Examples that are handled:
@@ -347,11 +357,71 @@
         return recipient;
       else
         return null;
-    }
+    };
+
+    this.setFromIdentity = function (identity) {
+      var node, children, nl, reNl, space, signature, previousIdentity;
+
+      if (identity && identity.full)
+        this.message.editable.from = identity.full;
+      else if (identity && identity.length)
+        return;
+
+      if (this.composeType == "html") {
+        nl = '<br />';
+        reNl = '<br ?/>[ \n]?';
+        space = '&nbsp;';
+      } else {
+        nl = '\n';
+        reNl = '\n';
+        space = ' ';
+      }
+
+      if (identity && identity.signature)
+        signature = nl + nl + '--' + space + nl + identity.signature;
+      else
+        signature = '';
+
+      previousIdentity = _.find(this.identities, function (currentIdentity, index) {
+        if (currentIdentity.signature) {
+          var currentSignature = new RegExp(reNl + reNl + '--' + space + reNl +
+                                            currentIdentity.signature.replace(/[-\[\]{}()*+?.,\\^$|#\s]/g, '\\$&'));
+          if (vm.message.editable.text.search(currentSignature) >= 0) {
+            vm.message.editable.text = vm.message.editable.text.replace(currentSignature, signature);
+            return true;
+          }
+        }
+        return false;
+      });
+
+      if (!previousIdentity && signature.length > 0) {
+        // Must place signature at proper place
+        if (!this.isNew() && this.signaturePlacement == 'above') {
+          var quotedMessageIndex = this.message.editable.text.search(new RegExp(reNl + '.+?:( ?' + reNl + '){2}(> |<blockquote type="cite")'));
+          if (quotedMessageIndex >= 0) {
+            this.message.editable.text =
+              this.message.editable.text.slice(0, quotedMessageIndex) +
+              signature +
+              this.message.editable.text.slice(quotedMessageIndex);
+          } else {
+            this.message.editable.text = signature + this.message.editable.text;
+          }
+        } else {
+          this.message.editable.text += signature;
+        }
+      }
+    };
+
+    this.identitySearch = function (query) {
+      var q = query ? query : '';
+      return _.filter(stateAccount.identities, function(identity) {
+        return identity.full.toLowerCase().indexOf(q.toLowerCase()) >= 0;
+      });
+    };
 
     this.expandGroup = function(contact, field) {
       var recipients, i, j;
-      recipients = vm.message.editable[field];
+      recipients = this.message.editable[field];
       i = recipients.indexOf(contact);
       recipients.splice(i, 1);
       for (j = 0; j < contact.members.length; j++) {
@@ -387,8 +457,7 @@
       if (this.firstFocus) {
         onCompletePromise().then(function(element) {
           var textContent = angular.element(textArea).val(),
-              hasSignature = (Preferences.defaults.SOGoMailSignature &&
-                              Preferences.defaults.SOGoMailSignature.length > 0),
+              hasSignature = /\n-- \n/.test(textContent),
               signatureLength = 0,
               sigLimit,
               caretPosition;
@@ -398,8 +467,9 @@
             element.find('md-dialog-content')[0].scrollTop = 0;
           }
           else {
+            // Search for signature starting from bottom
             if (hasSignature) {
-              sigLimit = textContent.lastIndexOf("--");
+              sigLimit = textContent.lastIndexOf("-- ");
               if (sigLimit > -1)
                 signatureLength = (textContent.length - sigLimit);
             }
@@ -415,14 +485,21 @@
       }
     };
 
-    this.onHTMLFocus = function ($event) {
-      var caretAtTop = (this.replyPlacement == 'above');
+    this.onHTMLReady = function ($editor) {
+      if (!this.isNew()) {
+        onCompletePromise().then(function() {
+          $editor.focus();
+        });
+      }
+    };
 
+    this.onHTMLFocus = function (editor) {
       if (this.firstFocus) {
         onCompletePromise().then(function(element) {
-          var selected = $event.editor.getSelection(),
+          var caretAtTop = (vm.replyPlacement == 'above'),
+              selected = editor.getSelection(),
               selected_ranges = selected.getRanges(),
-              children = $event.editor.document.getBody().getChildren(),
+              children = editor.document.getBody().getChildren(),
               node;
 
           if (caretAtTop) {
@@ -436,7 +513,7 @@
               if (x === null) {
                 break;
               }
-              if (x.getText() == '--') {
+              if (/--(%20|%A0|%C2%A0)/.test(encodeURI(x.getText()))) {
                 node = x.getPrevious().getPrevious();
                 break;
               }
