@@ -1,6 +1,7 @@
 /* -*- Mode: javascript; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 
 (function() {
+  /* jshint loopfunc: true */
   'use strict';
 
   /**
@@ -179,8 +180,30 @@
         data = {};
       }
 
-      // We convert our PreventInvitationsWhitelist hash into a array of user
       if (data.Calendar) {
+
+        // When the Calendar settings include "AutoReloadedWebCalendars", reload the Web calendars
+        // marked to be reloaded on login (AutoReloadedWebCalendars). Once completed, remove the
+        // parameter and save the settings.
+        if (data.Calendar.ReloadWebCalendars && data.Calendar.AutoReloadedWebCalendars) {
+          var reloadPromises = [];
+          _.map(data.Calendar.AutoReloadedWebCalendars, function (autoReload, id) {
+            if (autoReload) {
+              var calendarId = id.split('/')[1],
+                  deferred = Preferences.$q.defer();
+              Preferences.$$resource.quietFetch('Calendar/' + calendarId, 'reload').finally(deferred.resolve);
+              reloadPromises.push(deferred.promise);
+            }
+          });
+          Preferences.$q.all(reloadPromises).then(function() {
+            delete _this.settings.Calendar.ReloadWebCalendars;
+            Preferences.$$resource.save("Preferences", { settings: _this.$omit(true).settings }).then(function () {
+              Preferences.$rootScope.$emit('calendars:list');
+            });
+          });
+        }
+
+        // We convert our PreventInvitationsWhitelist hash into a array of user
         if (data.Calendar.PreventInvitationsWhitelist) {
           data.Calendar.PreventInvitationsWhitelist = _.map(data.Calendar.PreventInvitationsWhitelist, function(value, key) {
             var match = /^(.+)\s<(\S+)>$/.exec(value),
@@ -203,10 +226,11 @@
    * @desc The factory we'll use to register with Angular
    * @returns the Preferences constructor
    */
-  Preferences.$factory = ['$window', '$document', '$q', '$timeout', '$log', '$state', '$mdDateLocale', '$mdToast', 'sgSettings', 'Gravatar', 'Resource', 'User', function($window, $document, $q, $timeout, $log, $state, $mdDateLocaleProvider, $mdToast, Settings, Gravatar, Resource, User) {
+  Preferences.$factory = ['$window', '$document', '$rootScope', '$q', '$timeout', '$log', '$state', '$mdDateLocale', '$mdToast', 'sgSettings', 'Gravatar', 'Resource', 'User', function($window, $document, $rootScope, $q, $timeout, $log, $state, $mdDateLocaleProvider, $mdToast, Settings, Gravatar, Resource, User) {
     angular.extend(Preferences, {
       $window: $window,
       $document: $document,
+      $rootScope: $rootScope,
       $q: $q,
       $timeout: $timeout,
       $log: $log,
@@ -386,38 +410,50 @@
     if (this.nextInboxPoll)
       Preferences.$timeout.cancel(this.nextInboxPoll);
 
-    Preferences.$$resource.post('Mail', '0/folderINBOX/view', params).then(function(data) {
-      var uids = data.uids;
-      var uidHeaderIndex = data.headers[0].indexOf('uid');
-      var fromHeaderIndex = data.headers[0].indexOf('From');
-      var subjectHeaderIndex = data.headers[0].indexOf('Subject');
-      if (data.threaded) {
-        data.uids.splice(0, 1);
-        uids = _.map(data.uids, 0);
+    if (this.inboxSyncToken)
+      params.syncToken = this.inboxSyncToken;
+
+    Preferences.$$resource.post('Mail', '0/folderINBOX/changes', params).then(function(data) {
+      if (data.syncToken) {
+        _this.inboxSyncToken = data.syncToken;
+        Preferences.$log.debug("New syncToken is " + _this.inboxSyncToken);
       }
-      if (_this.lastUid) {
-        _.find(uids, function (uid, index) {
-          var headers, id, href, toast;
-          if (uid > _this.lastUid) {
+
+      if (angular.isDefined(data.headers) && data.headers.length > 0) {
+        var uidHeaderIndex = data.headers[0].indexOf('uid');
+        var isReadHeaderIndex = data.headers[0].indexOf('isRead');
+        var fromHeaderIndex = data.headers[0].indexOf('From');
+        var subjectHeaderIndex = data.headers[0].indexOf('Subject');
+        var i;
+        var showToast = function() {
+          var _this = this;
+          return Preferences.$toast.show(this)
+            .then(function(response) {
+              if (response === 'ok') {
+                _this.viewInboxMessage(_this.locals.uid);
+              }
+            });
+        };
+        for (i = 1; i < data.headers.length; i++) {
+          var headers = data.headers[i],
+              uid = headers[uidHeaderIndex],
+              id, href, toast;
+          if (!headers[isReadHeaderIndex]) {
             // New unseen message
             Preferences.$log.debug('Show notification for message ' + uid);
-            headers = _.find(data.headers, function(h) {
-              return h[uidHeaderIndex] == uid;
-            });
             if (_this.defaults.SOGoDesktopNotifications) {
               id = 'mail-inbox-' + uid;
               href = Preferences.$state.href('mail.account.mailbox.message', { accountId: 0, mailboxId: 'INBOX', messageId: uid });
               _this.createNotification(id, headers[subjectHeaderIndex], {
                 body: headers[fromHeaderIndex][0].name || headers[fromHeaderIndex][0].email,
                 icon: '/SOGo.woa/WebServerResources/img/email-256px.png',
-                onClick: function () {
-                  _this.viewInboxMessage(uid);
-                }
+                onClick: angular.bind(_this, _this.viewInboxMessage, uid)
               });
             }
             else {
               toast = {
                 locals: {
+                  uid: uid,
                   title: headers[subjectHeaderIndex],
                   body: headers[fromHeaderIndex][0].name || headers[fromHeaderIndex][0].email
                 },
@@ -440,28 +476,13 @@
                 ].join(''),
                 position: 'top right',
                 hideDelay: 5000,
-                controller: toastController
+                controller: toastController,
+                viewInboxMessage: _this.viewInboxMessage
               };
-              _this.currentToast = _this.currentToast.then(function () {
-                return Preferences.$toast.show(toast)
-                .then(function(response) {
-                  if (response === 'ok') {
-                    _this.viewInboxMessage(uid);
-                  }
-                });
-              });
+              _this.currentToast = _this.currentToast.then(angular.bind(toast, showToast));
             }
-            return false; // Continue to next unseen message
           }
-          else {
-            return true; // No more new messages
-          }
-        });
-        if (uids[0] > _this.lastUid) {
-          _this.lastUid = uids[0];
         }
-      } else {
-        _this.lastUid = uids[0];
       }
     }).finally(function () {
       var refreshViewCheck = _this.defaults.SOGoRefreshViewCheck;
@@ -629,7 +650,7 @@
           Preferences.$toast.hide();
         };
         scope.snooze = function() {
-          Preferences.$$resource.fetch(url, 'view?snoozeAlarm=' + scope.reminder);
+          Preferences.$$resource.fetch('Calendar/' + url, 'view?snoozeAlarm=' + scope.reminder);
           Preferences.$toast.hide();
         };
       }
@@ -674,6 +695,9 @@
       }
     });
 
+    // Don't push locale definition
+    delete preferences.defaults.locale;
+
     // Merge back mail labels keys and values
     preferences.defaults.SOGoMailLabelsColors = {};
     _.forEach(preferences.defaults.SOGoMailLabelsColorsKeys, function(key, i) {
