@@ -483,7 +483,7 @@
 
     return new Date(parseInt(date[0]), parseInt(date[1]) - 1, parseInt(date[2]),
                     parseInt(time[0]), parseInt(time[1]), 0, 0);
-    };
+  };
 
   /**
    * @function init
@@ -599,9 +599,9 @@
       this.repeat.end = 'never';
     this.$hasCustomRepeat = this.hasCustomRepeat();
 
+    var type = (this.type == 'appointment')? 'Events' : 'Tasks';
     if (this.isNew) {
       // Set default values
-      var type = (this.type == 'appointment')? 'Events' : 'Tasks';
 
       // Set default classification
       this.classification = Component.$Preferences.defaults['SOGoCalendar' + type + 'DefaultClassification'].toLowerCase();
@@ -618,8 +618,13 @@
       // Set notitifications
       this.sendAppointmentNotifications = Component.$Preferences.defaults.SOGoAppointmentSendEMailNotifications;
     }
-    else if (angular.isUndefined(data.$hasAlarm)) {
-      this.$hasAlarm = angular.isDefined(data.alarm);
+    else {
+      if (angular.isUndefined(data.$hasAlarm)) {
+        this.$hasAlarm = angular.isDefined(data.alarm);
+      }
+      if (angular.isUndefined(data.classification)) {
+        this.classification = Component.$Preferences.defaults['SOGoCalendar' + type + 'DefaultClassification'].toLowerCase();
+      }
     }
 
     // Allow the component to be moved to a different calendar
@@ -941,7 +946,8 @@
     data = {
       reply: this.reply,
       delegatedTo: this.delegatedTo,
-      alarm: this.$hasAlarm? this.alarm : {}
+      alarm: this.$hasAlarm? this.alarm : {},
+      classification: this.classification
     };
 
     return Component.$$resource.save(path, data, { action: 'rsvpAppointment' })
